# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '28/Oct/2018'
__author__ = 'Erdinç Yılmaz'
__license__ = 'GPLv3'

import os
from PySide6.QtGui import (QPen, QBrush, QPainterPath, QPainterPathStroker, QColor, QTextOption, QPixmap, QFont, QDrag)
from PySide6.QtWidgets import (QGraphicsItem, QStyle, QGraphicsPixmapItem, QFileIconProvider)
from PySide6.QtCore import (QRectF, Qt, QMimeData, QUrl, QFileInfo, QSize)
from .. import shared


########################################################################
class KutuphaneNesnesi(QGraphicsItem):
    Type = shared.KUTUPHANE_NESNESI

    def __init__(self, pos, tip, dosya_adresi, sahnede_kullaniliyor_mu=True,
                 belgede_kullaniliyor_mu=True,
                 isEmbeded=True,
                 isHtmlImage=False,
                 parent=None):
        super(KutuphaneNesnesi, self).__init__(parent)
        # QGraphicsItem.__init__(self, parent,scene)

        self.sahnede_kullaniliyor_mu = sahnede_kullaniliyor_mu
        self.belgede_kullaniliyor_mu = belgede_kullaniliyor_mu
        self.isEmbeded = isEmbeded
        self.isHtmlImage = isHtmlImage

        self.tip = tip

        self.dosya_adresi = dosya_adresi

        if os.path.isfile(dosya_adresi):
            self.filePathForDraw = dosya_adresi
        else:
            self.filePathForDraw = ':icons/warning.png'

        self._pixmap_olustur()

        rectf = QRectF(self.pixmap.rect())

        self.onizlemePixmapItem = None

        self.setPos(pos)
        self._rect = rectf
        self._boundingRect = QRectF()

        self.setAcceptHoverEvents(True)

        self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable)

        # self.setFlags(QGraphicsItem.ItemIsSelectable |
        #               # QGraphicsItem.ItemIsMovable |
        #               QGraphicsItem.ItemIsFocusable)

        # self.setFiltersChildEvents(True)
        # self.setHandlesChildEvents(True)

        self._pen = QPen()
        self._brush = QBrush()
        # TODO: fonta gerek var mi
        self._font = QFont()
        self._font.setBold(True)
        # self._font.setPointSize(12)
        # self._text = dosya_adresi

        self.painterTextOption = QTextOption()
        self.painterTextOption.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.painterTextOption.setWrapMode(QTextOption.WrapAtWordBoundaryOrAnywhere)

        self.activeItemLineColor = shared.activeItemLineColor
        # self.cizgiRengi = pen.color()
        # self.setCizgiRengi(self.cizgiRengi)  # also sets self._pen
        self.setCizgiRengi(QColor(Qt.GlobalColor.red))
        # self.yaziRengi = yaziRengi
        # self.setYaziRengi(yaziRengi)
        self.setYaziRengi(QColor(Qt.GlobalColor.black))
        self.setArkaPlanRengi(QColor(Qt.GlobalColor.transparent))  # also sets self._brush # veya nobrush

        self.setToolTip(f'{dosya_adresi}\n\n -- Press "Alt" to preview"')

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return KutuphaneNesnesi.Type

        # ---------------------------------------------------------------------
    def _pixmap_olustur(self):

        if self.tip == shared.VIDEO_ITEM_TYPE or self.tip == shared.DOSYA_ITEM_TYPE:
            iconProvider = QFileIconProvider()
            self.ikon = iconProvider.icon(QFileInfo(self.dosya_adresi))
            # self.pixmap = self.ikon.pixmap(self._rect.toRect().size())
            self.pixmap = self.ikon.pixmap(QSize(45, 45))
        else:
            self.pixmap = QPixmap(self.filePathForDraw).scaledToWidth(45, Qt.TransformationMode.FastTransformation)


    # # ---------------------------------------------------------------------
    # def _pixmap_olustur_eski(self):
    #
    #
    #     # # self.pixmap = pixmap
    #     # self.pixmap = QPixmap()
    #     # if not QPixmapCache.find(self.filePathForDraw, self.pixmap):
    #     #     self.pixmap = QPixmap(self.filePathForDraw)
    #     #     QPixmapCache.insert(self.filePathForDraw, self.pixmap)
    #     #
    #     # # self.pixmap = QPixmap(self.filePathForDraw)
    #
    #     # self.pixmap = pixmap
    #     # self.pixmap = QPixmap()
    #     # print(QPixmapCache.cacheLimit())
    #     # self.pixmap = QPixmapCache.find(self.filePathForDraw, self.pixmap)
    #     self.pixmap = QPixmapCache.find(self.filePathForDraw)
    #     if not self.pixmap:
    #         # print("asdasd")
    #         # viewRectSize = self.scene().views()[0].get_visible_rect().size().toSize()
    #         # self.pixMap = QPixmap(self.filePathForDraw).scaled(viewRectSize / 1.5, Qt.KeepAspectRatio)
    #         if self.tip == shared.VIDEO_ITEM_TYPE or self.tip == shared.DOSYA_ITEM_TYPE:
    #             iconProvider = QFileIconProvider()
    #             self.ikon = iconProvider.icon(QFileInfo(dosya_adresi))
    #             # self.pixmap = self.ikon.pixmap(self._rect.toRect().size())
    #             self.pixmap = self.ikon.pixmap(QSize(45, 45))
    #         else:
    #             self.pixmap = QPixmap(self.filePathForDraw).scaledToWidth(45, Qt.TransformationMode.FastTransformation)
    #         QPixmapCache.insert(self.filePathForDraw, self.pixmap)
    #     else:
    #         self.pixmap = self.pixmap.scaledToWidth(45, Qt.TransformationMode.FastTransformation)

    # ---------------------------------------------------------------------
    def _onizleme_pixmap_olustur(self):
        # viewRectSize = self.scene().views()[0].get_visible_rect().size().toSize()
        # self.pixMap = QPixmap(self.filePathForDraw).scaled(viewRectSize / 1.5, Qt.KeepAspectRatio)
        self.onizlemePixmap = QPixmap(self.filePathForDraw).scaledToWidth(300, Qt.TransformationMode.FastTransformation)

        self.onizlemePixmapItem = QGraphicsPixmapItem(self.onizlemePixmap, self)

    # # ---------------------------------------------------------------------
    # def _onizleme_pixmap_olustur_eski(self):
    #     self.onizlemePixmap = QPixmapCache.find(self.dosya_adresi)
    #     if not self.onizlemePixmap:
    #         # print("asdasd")
    #         # viewRectSize = self.scene().views()[0].get_visible_rect().size().toSize()
    #         # self.pixMap = QPixmap(self.filePathForDraw).scaled(viewRectSize / 1.5, Qt.KeepAspectRatio)
    #         self.onizlemePixmap = QPixmap(self.filePathForDraw).scaledToWidth(300, Qt.TransformationMode.FastTransformation)
    #         QPixmapCache.insert(self.dosya_adresi, self.onizlemePixmap)
    #     else:
    #         self.onizlemePixmap = self.onizlemePixmap.scaledToWidth(300, Qt.TransformationMode.FastTransformation)
    #
    #     self.onizlemePixmapItem = QGraphicsPixmapItem(self.onizlemePixmap, self)

    # ---------------------------------------------------------------------
    def sceneCenter(self):
        return self.mapToScene(self._rect.center())

    # ---------------------------------------------------------------------
    def sceneWidth(self):
        return self.sceneRight() - self.sceneLeft()

    # ---------------------------------------------------------------------
    def sceneHeight(self):
        return self.sceneBottom() - self.sceneTop()

    # ---------------------------------------------------------------------
    def sceneRight(self):
        return max(self.mapToScene(self._rect.topLeft()).x(),
                   self.mapToScene(self._rect.topRight()).x(),
                   self.mapToScene(self._rect.bottomRight()).x(),
                   self.mapToScene(self._rect.bottomLeft()).x())

    # ---------------------------------------------------------------------
    def sceneLeft(self):
        return min(self.mapToScene(self._rect.topLeft()).x(),
                   self.mapToScene(self._rect.topRight()).x(),
                   self.mapToScene(self._rect.bottomRight()).x(),
                   self.mapToScene(self._rect.bottomLeft()).x())

    # ---------------------------------------------------------------------
    def sceneTop(self):
        return min(self.mapToScene(self._rect.topLeft()).y(),
                   self.mapToScene(self._rect.topRight()).y(),
                   self.mapToScene(self._rect.bottomRight()).y(),
                   self.mapToScene(self._rect.bottomLeft()).y())

    # ---------------------------------------------------------------------
    def sceneBottom(self):
        return max(self.mapToScene(self._rect.topLeft()).y(),
                   self.mapToScene(self._rect.topRight()).y(),
                   self.mapToScene(self._rect.bottomRight()).y(),
                   self.mapToScene(self._rect.bottomLeft()).y())

    # ---------------------------------------------------------------------
    def setSceneLeft(self, left):
        self.setX(left + self.scenePos().x() - self.sceneLeft())

    # ---------------------------------------------------------------------
    def setSceneRight(self, right):
        self.setX(right + self.scenePos().x() - self.sceneRight())

    # ---------------------------------------------------------------------
    def setSceneTop(self, top):
        self.setY(top + self.scenePos().y() - self.sceneTop())

    # ---------------------------------------------------------------------
    def setSceneBottom(self, bottom):
        self.setY(bottom + self.scenePos().y() - self.sceneBottom())

    # ---------------------------------------------------------------------
    def setHorizontalCenter(self, hcenter):
        self.setY(hcenter + self.scenePos().y() - self.sceneCenter().y())

    # ---------------------------------------------------------------------
    def setVerticalCenter(self, vcenter):
        self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x())

    # # ---------------------------------------------------------------------
    # def setVerticalCenterWhenParented(self, vcenter):
    #     self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x() - self.parentItem().pos().x())

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return
        self.prepareGeometryChange()
        self._rect = rect
        self._boundingRect = QRectF()
        self.update()

    # ---------------------------------------------------------------------
    def rect(self):
        return self._rect

    # ---------------------------------------------------------------------
    def boundingRect(self):
        # TODO: kare gruplandiginda bu padding belli oluyor.
        # cunku self.rect ile paint ediyoruz. ellipsete yok bu problem.
        # pad = self.pen().widthF() / 2 + self.handleSize
        pad = self.pen().widthF() / 2
        self._boundingRect = QRectF(self._rect)
        return self._boundingRect.adjusted(-pad, -pad, pad, pad)

        # path = QPainterPath()
        # path.addRect(self._rect)
        # # path.moveTo(0,0)
        # return path.boundingRect()

    # ---------------------------------------------------------------------
    def shape(self):
        path = QPainterPath()
        # path.addRect(self.rect)
        path.addRect(self._rect)
        return self.qt_graphicsItem_shapeFromPath(path, self._pen)

    # ---------------------------------------------------------------------
    def qt_graphicsItem_shapeFromPath(self, path, pen):

        # We unfortunately need this hack as QPainterPathStroker will set a width of 1.0
        # if we pass a value of 0.0 to QPainterPathStroker.setWidth()
        penWidthZero = 0.00000001
        if path == QPainterPath():
            return path
        ps = QPainterPathStroker()
        ps.setCapStyle(pen.capStyle())

        if pen.widthF() <= 0:
            ps.setWidth(penWidthZero)
        else:
            ps.setWidth(pen.widthF())
        ps.setJoinStyle(pen.joinStyle())
        ps.setMiterLimit(pen.miterLimit())
        p = ps.createStroke(path)  # return type: QPainterPath
        p.addPath(path)
        return p

    # ---------------------------------------------------------------------
    # TODO: maybe: to: self.paint(): if option.state & QStyle.State_Selected:
    # const QRectF murect = painter->transform().mapRect(QRectF(0, 0, 1, 1));
    # if (qFuzzyIsNull(qMax(murect.width(), murect.height())))
    #     return;
    #
    # const QRectF mbrect = painter->transform().mapRect(item->boundingRect());
    # if (qMin(mbrect.width(), mbrect.height()) < qreal(1.0))
    #     return;

    # ---------------------------------------------------------------------
    def setFont(self, font):
        font.setBold(self._font.bold())
        font.setItalic(self._font.italic())
        font.setUnderline(self._font.underline())
        font.setStrikeOut(self._font.strikeOut())
        font.setOverline(self._font.overline())
        font.setPointSizeF(self._font.pointSizeF())
        self._font = font
        self.update()

    # ---------------------------------------------------------------------
    def font(self):
        return self._font

    # ---------------------------------------------------------------------
    def setFontPointSizeF(self, fontPointSizeF):
        self._font.setPointSizeF(fontPointSizeF)
        self.update()

    # ---------------------------------------------------------------------
    def fontPointSizeF(self):
        return self._font.pointSizeF()

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def setBrush(self, brush):

        if self._brush == brush:
            return
        self._brush = brush
        self.update()

    # ---------------------------------------------------------------------
    def brush(self):
        return self._brush

    # # ---------------------------------------------------------------------
    # def setText(self, text):
    #     self._text = text
    #     self.update()
    #
    # # ---------------------------------------------------------------------
    # def text(self):
    #     return self._text

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base- text -path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        # self.scene().parent().on_item_context_menu_about_to_show(self)
        # self.scene().parent().itemContextMenu.popup(event.screenPos())
        self.scene().parent().kutuphaneNesneSagMenu.popup(event.screenPos())
        event.accept()

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):
        """sahneden nesneleri secebilir"""

        super(KutuphaneNesnesi, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):

        # _selectionLineBgColor = QColor.fromHsv(col.hue(),
        #                                        0,
        #                                        20 if col.value() > 127 else 250)

        _activeItemLineColor = QColor(self.activeItemLineColor)
        if col.hue() > 300 or col.hue() < 30:
            _activeItemLineColor.setHsv((col.hue() + 150) % 360,
                                        self.activeItemLineColor.saturation(),
                                        self.activeItemLineColor.value())

        self._pen.setColor(col)
        self.cizgiRengi = col

        # self.selectionPenBottom = QPen(_selectionLineBgColor,
        #                                self._pen.width(),
        #                                Qt.DashLine,
        #                                Qt.FlatCap, Qt.RoundJoin)

        self.selectionPenBottom = QPen(_activeItemLineColor,
                                       self._pen.width(),
                                       Qt.PenStyle.DashLine,
                                       Qt.PenCapStyle.FlatCap, Qt.PenJoinStyle.RoundJoin)

        self.update()

    # ---------------------------------------------------------------------
    def setYaziRengi(self, col):

        # drawing text after drawing rect does not apply alpha
        # we need to reconstruct the color with same values.
        self.yaziRengi = col
        # self.textPen = QPen(QColor().fromRgb(col.red(), col.green(), col.blue(), col.alpha()))
        self.textPen = QPen(col)
        self.update()

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        self.arkaPlanRengi = col
        self.setBrush(QBrush(col))

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(KutuphaneNesnesi, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, QGraphicsSceneHoverEvent):
        # TODO: bu daha optimize edilebilir hatta iimage ve bunun normal pixmapleri de
        # scale kısmı verimli degil gibi
        # scale edilmişi de mi cache atıyoruz.
        # else kısmında ve de.

        if QGraphicsSceneHoverEvent.modifiers() == Qt.KeyboardModifier.AltModifier:

            # self.pixmap = QPixmap()
            # print(QPixmapCache.cacheLimit())
            # self.pixmap = QPixmapCache.find(self.filePathForDraw, self.pixmap)
            if not self.onizlemePixmapItem:
                self._onizleme_pixmap_olustur()
            else:
                self.onizlemePixmapItem.setVisible(True)

            self.scene().zDeger_arttir(self)
        super(KutuphaneNesnesi, self).hoverEnterEvent(QGraphicsSceneHoverEvent)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, QGraphicsSceneHoverEvent):
        if self.onizlemePixmapItem:
            self.onizlemePixmapItem.setVisible(False)
        super(KutuphaneNesnesi, self).hoverLeaveEvent(QGraphicsSceneHoverEvent)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, e):
        # if e.buttons() != Qt.RightButton:
        #     return

        # write the relative cursor position to mime data
        mimeData = QMimeData()
        # simple string with 'x,y'
        # mimeData.setText('%d,%d' % (e.x(), e.y()))
        mimeData.setUrls((QUrl.fromLocalFile(self.dosya_adresi),))

        # # below makes the pixmap half transparent
        # painter = QPainter(pixmap)
        # painter.setCompositionMode(painter.CompositionMode_DestinationIn)
        # painter.fillRect(pixmap.rect(), QColor(0, 0, 0, 127))
        # painter.end()

        # make a QDrag
        drag = QDrag(e.widget())
        # put our MimeData
        drag.setMimeData(mimeData)
        # set its Pixmap
        drag.setPixmap(self.pixmap)
        # shift the Pixmap so that it coincides with the cursor position
        drag.setHotSpot(e.pos().toPoint())

        # start the drag operation
        # exec_ will return the accepted action from dropEvent
        if drag.exec_(Qt.DropAction.CopyAction | Qt.DropAction.MoveAction) == Qt.DropAction.MoveAction:
            print('moved')
            self.sahnede_kullaniliyor_mu = True
            self.belgede_kullaniliyor_mu = True
            self.update()
        else:
            print('copied')

        self.setCursor(Qt.CursorShape.OpenHandCursor)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        self.setCursor(Qt.CursorShape.OpenHandCursor)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        self.setCursor(Qt.CursorShape.ClosedHandCursor)
        # QGraphicsItem.mousePressEvent(self, e)
        # if e.button() == Qt.LeftButton:
        #     print('press')

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):

        painter.setClipRect(option.exposedRect)

        painter.setFont(self._font)
        painter.setBrush(Qt.BrushStyle.NoBrush)
        # rect = self.boundingRect().toRect()
        rect = self._rect.toRect()
        painter.drawPixmap(rect, self.pixmap)

        # # painter.setWorldMatrixEnabled(False)
        # painter.save()
        # metrics = painter.fontMetrics()
        # # --- basla -- elided text icin-------------------------------------------------------------
        # # txt = metrics.elidedText(self._text, Qt.ElideRight, self._rect.width() / scale)
        # # r = QRectF(metrics.boundingRect(txt))
        # # --- bitir -- elided text icin-------------------------------------------------------------
        #
        # r = QRectF(metrics.boundingRect(self._rect.toRect(), Qt.TextWrapAnywhere, self._text))
        # r.moveCenter(self._rect.center())
        # # draw text background rect
        # painter.setPen(Qt.NoPen)
        # painter.setBrush(self.arkaPlanRengi)
        #
        # painter.drawRect(r.intersected(self._rect))
        # # draw text
        # # painter.setPen(self._pen)
        # painter.setPen(self.yaziRengi)
        # # painter.drawText(self._rect, Qt.AlignCenter, txt)
        # painter.drawText(self._rect, self._text, self.painterTextOption)
        # # painter.drawText(self.painterTextRect, self._text, self.painterTextOption)
        #
        # painter.restore()
        # # painter.setWorldMatrixEnabled(True)

        if option.state & QStyle.StateFlag.State_Selected:
            selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            painter.setPen(selectionPenBottom)
            painter.drawRect(self._rect)

            # painter.setPen(self.selectionPenTop)
            # painter.drawRect(self._rect)

        if self.isHtmlImage:
            painter.setBrush(Qt.GlobalColor.cyan)
            painter.setPen(Qt.GlobalColor.black)
            painter.drawEllipse(QRectF(0, 0, 16, 15))
            painter.drawText(QRectF(3, 0, 16, 15), "w")
            return

        if not self.isEmbeded:
            painter.setBrush(Qt.GlobalColor.yellow)
            painter.setPen(Qt.GlobalColor.black)
            painter.drawEllipse(QRectF(0, 0, 15, 15))
            painter.drawText(QRectF(3, 0, 15, 15), "~")
            return

        if not self.belgede_kullaniliyor_mu:
            painter.setBrush(Qt.GlobalColor.red)
            painter.setPen(Qt.GlobalColor.white)
            painter.drawEllipse(QRectF(0, 0, 15, 15))
            painter.drawText(QRectF(3, 0, 15, 15), "X")
            # painter.drawEllipse(QRectF(0,0,10,10))
            return

        if not self.sahnede_kullaniliyor_mu:
            painter.setBrush(Qt.GlobalColor.darkYellow)
            painter.setPen(Qt.GlobalColor.white)
            painter.drawEllipse(QRectF(0, 0, 15, 15))
            painter.drawText(QRectF(3, 0, 15, 15), "X")
            # painter.drawEllipse(QRectF(0,0,10,10))
            return



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '01/Jun/2018'
__author__ = 'Erdinç Yılmaz'

import tempfile
from typing import Any

from PySide6.QtCore import QAbstractItemModel, QModelIndex, Qt, Signal, QSize, QMimeData, QByteArray

from .treeSayfa import Sayfa
from .shared import renk_degismis_nesne_yazi_rengi


# dataChanged : belirli row veya columnları gunceller , ekleme cıkarma yok
# layoutAboutToBeChanged , layoutChanged, bunlar viewa her şeyi gunclletir, tekrar çizdirir, row column ekleme için
# layoutAboutToBeChanged ve turevlerini mumkun oldugunca biz emit etmiyoruz,
# beginInsert, endInsert kullanaraktan bunlar emit ediyor gerektigi kadarı ile gerektigi yerde. tabi bu bir tavsiye

########################################################################
class DokumanAgacModel(QAbstractItemModel):
    dokuman_no = 1

    MimeType = "object/x-standart-item"

    nesneDegisti = Signal(Sayfa, str)

    # ---------------------------------------------------------------------
    def __init__(self, tempDirPath=None, parent=None):
        super(DokumanAgacModel, self).__init__(parent)

        # bunların fonksiyonları scene icinde
        # her sayfa scene , ayrı oldugu icin aynı dokuman icinde farklı sayfalarda uzerine yazma
        # durumu oluyordu.

        self.embededImageCounter = 0
        self.embededVideoCounter = 0
        self.embededFileCounter = 0

        self.treeViewIkonBoyutu = QSize(48, 48)
        self.yaziRengi = parent.palette().windowText().color()

        # self.sayfalar = []
        Sayfa.sayfa_no = 0  # her dokuman icin sayfa sayisini sifirliyoruz
        self.kokSayfa = Sayfa()  # omurga
        self.kokSayfa.adi = "kokSayfa"
        # self.sayfa_ekle(self.kokSayfa, ustSayfa=None)
        # self.

        if not tempDirPath:
            tempDirPath = tempfile.mkdtemp(prefix="defter-tmp")

        self.tempDirPath = tempDirPath
        self.saveFilePath = ""
        self.no = DokumanAgacModel.dokuman_no
        self.fileName = self.tr("Untitled Document - {}".format(self.no))

        self.enSonAktifSayfa = None

        DokumanAgacModel.dokuman_no += 1

        # self._roleNames = {}
        # for i in range(0, meta.propertyCount()):
        #     self._roleNames[i] = meta.property(i).name()

    # ---------------------------------------------------------------------
    def degismis_sayfa_var_mi(self):

        var_mi = False

        for sayfa in self.sayfalar():
            if not sayfa.scene.undoStack.isClean():
                var_mi = True
                break

        return var_mi

    # ---------------------------------------------------------------------
    def degismis_sayfalarin_listesi(self):
        liste = []
        for sayfa in self.sayfalar():
            if not sayfa.scene.undoStack.isClean():
                liste.append(sayfa)

        return liste

    # ---------------------------------------------------------------------
    def iterItems(self, root):
        if root is not None:
            stack = [root]
            while stack:
                parent = stack.pop(0)
                # for row in range(parent.satirSayisi()):
                #     for column in range(parent.sutunSayisi()):
                #         child = parent.child(row, column)
                #         yield child
                #         if child.hasChildren():
                #             stack.append(child)
                for row in range(parent.satirSayisi()):
                    ic_sayfa = parent.ic_sayfa(row)
                    yield ic_sayfa
                    if ic_sayfa.ic_sayfa_var_mi():
                        stack.append(ic_sayfa)

    # ---------------------------------------------------------------------
    def kaydedince_butun_yildizlari_sil(self):
        self._kaydedince_butun_yildizlari_sil(self.kokSayfa.ic_sayfalar())

    # ---------------------------------------------------------------------
    def _kaydedince_butun_yildizlari_sil(self, sayfalar):
        for sayfa in sayfalar:
            sayfa.scene.blockSignals(True)
            sayfa.scene.undoStack.setClean()

            itemText = sayfa.adi
            if itemText[0] == "★":
                sayfa.adi = itemText[2:]

            idx = self.index_sayfadan(sayfa)
            self.dataChanged.emit(idx, idx)

            sayfa.scene.blockSignals(False)

            if sayfa._ic_sayfalar:
                self._kaydedince_butun_yildizlari_sil(sayfa._ic_sayfalar)

    # ---------------------------------------------------------------------
    def sayfalar(self):
        # TODO: bu sıra gozetmiyor
        stack = [self.kokSayfa]
        while stack:
            parent = stack.pop(0)
            # for row in range(parent.satirSayisi()):
            #     for column in range(parent.sutunSayisi()):
            #         child = parent.child(row, column)
            #         yield child
            #         if child.hasChildren():
            #             stack.append(child)
            for row in range(parent.satirSayisi()):
                ic_sayfa = parent.ic_sayfa(row)
                yield ic_sayfa
                if ic_sayfa.ic_sayfa_var_mi():
                    stack.append(ic_sayfa)

    # ---------------------------------------------------------------------
    def get_properties_for_save(self):
        properties = {"saveFilePath": self.saveFilePath,
                      "fileName": self.fileName,
                      "embededImageCounter": self.embededImageCounter,
                      "embededVideoCounter": self.embededVideoCounter,
                      "embededFileCounter": self.embededFileCounter,
                      "enSonAktifSayfaKim": self.enSonAktifSayfa._kim,
                      "treeViewIkonBoyutu": self.treeViewIkonBoyutu
                      }
        return properties

    # ---------------------------------------------------------------------
    def kimlikten_sayfa(self, kim):
        for sayfa in self.sayfalar():
            if kim == sayfa._kim:
                # break
                return sayfa
        return None

    # ---------------------------------------------------------------------
    def sayfa_indexten(self, index):
        if index.isValid():
            return index.internalPointer()
            # if sayfa:
            #     return sayfa
            # return sayfa

        return self.kokSayfa

    # ---------------------------------------------------------------------
    def satirdaki_sayfa(self, satir):
        return self.kokSayfa.ic_sayfa(satir)

    # ---------------------------------------------------------------------
    def indexFromItem(self, item, column=0):
        if item == self.kokSayfa:
            return QModelIndex()

        parent = item.parent()
        if not parent:
            parent = self.kokSayfa

        if len(parent.children()) == 0:
            return None

        # print(item.title(), [i.title() for i in parent.children()])

        row = parent.children().index(item)
        col = column

        return self.createIndex(row, col, item)

    # ---------------------------------------------------------------------
    def index_sayfadan(self, sayfa, column=0):
        if not sayfa:
            return QModelIndex()

        if sayfa == self.kokSayfa:
            return QModelIndex()

        ustSayfa = sayfa.parent()
        if not ustSayfa:
            ustSayfa = self.kokSayfa

        if ustSayfa.satirSayisi() == 0:
            return None  # ?? hmm, bos index root olur tabi, none dondurmek lazım

        row = sayfa.satir()

        return self.createIndex(row, column, sayfa)

        # return self.index(row=sayfa.satir(), column=0, parent=sayfa.parent())

    # ---------------------------------------------------------------------
    def parent(self, index=None):
        """bu tamam olmasi lazım"""
        if not index.isValid():
            return QModelIndex()

        sayfa = index.internalPointer()
        ust_sayfa = sayfa.parent()

        # TODO: bu olmamıs olabilir.
        # yani tab sonra az saga kaydır yeni tab az saga kaydir yeni tab ve de burda ust_sayfa_none diyor
        # ama bundan sonra da problem var..
        if not ust_sayfa:
            return QModelIndex()

        if ust_sayfa == self.kokSayfa:
            return QModelIndex()

        return self.createIndex(ust_sayfa.satir(), 0, ust_sayfa)  # row, column, parent

    # ---------------------------------------------------------------------
    def index(self, row: int, column: int, parentIndex: QModelIndex = QModelIndex()):

        """İnş Tamam"""

        # allttaki de aktif edilebilir standatitem e edilmiş

        # if row < 0 or column < 0 or row >= parent.satirSayisi() or column >= parent.sutunSayisi():
        #     return QModelIndex()

        # parentNode = self.index_sayfadan(parent)

        # if parentIndex.isValid() and parentIndex.column() != 0:
        #     return QModelIndex()

        if not self.hasIndex(row, column, parentIndex):
            return QModelIndex()

        # parentItem = self.sayfa_indexten(parentIndex)
        # altakinin aynisi zaten func cagirmasin bosuna diye burda acik ekledik

        if not parentIndex.isValid():
            parentItem = self.kokSayfa
        else:
            parentItem = parentIndex.internalPointer()

        ic_sayfa = parentItem.ic_sayfa(row)
        if ic_sayfa:
            return self.createIndex(row, column, ic_sayfa)
        else:
            return QModelIndex()

    # ---------------------------------------------------------------------
    def supportedDropActions(self):
        # return Qt.CopyAction | Qt.MoveAction
        return Qt.DropAction.MoveAction

    # ---------------------------------------------------------------------
    def supportedDragActions(self):
        return Qt.DropAction.MoveAction

    # ---------------------------------------------------------------------
    # def moveRows(self, QModelIndex, p_int, p_int_1, QModelIndex_1, p_int_2):
    def moveRows(self, sourceParent, sourceRow, count, hedefParentIndex, hedefRow):
        kaynak_ust_sayfa = self.sayfa_indexten(sourceParent)
        hedef_ust_sayfa = self.sayfa_indexten(hedefParentIndex)

        # print(kaynak_ust_sayfa.adi,kaynak_ust_sayfa.satirSayisi(),
        # sourceRow, "--", hedef_ust_sayfa.satirSayisi(), hedefRow)
        tasinan_sayfa = kaynak_ust_sayfa.ic_sayfa(sourceRow)

        if hedefRow == -11:  # on item
            hedefRowYeni = hedef_ust_sayfa.satirSayisi() - 1
            # if hedef_ust_sayfa == kaynak_ust_sayfa:
            # print(sourceRow, hedefRow, hedefRowYeni)
            self.removeRow(sourceRow, sourceParent)
            # self.beginMoveRows(sourceParent, sourceRow, count, hedefParentIndex, hedefRowYeni)
            # kaynak_ust_sayfa.sayfa_sil(tasinan_sayfa)
            self.beginInsertRows(hedefParentIndex, hedefRowYeni, hedefRowYeni)
            hedef_ust_sayfa.sayfa_ekle(tasinan_sayfa)
            self.endInsertRows()

        elif hedefRow == -22:  # on view port
            hedefRowYeni = hedef_ust_sayfa.satirSayisi() - 1
            if hedef_ust_sayfa == kaynak_ust_sayfa:
                if hedefRowYeni == sourceRow:  # o seviyedeki son nesne ayni kendi yerine birakilinca
                    return
                # self.beginMoveRows(sourceParent, sourceRow, count, hedefParentIndex, hedefRowYeni)
                # hedef_ust_sayfa.sayfa_sira_degistir(tasinan_sayfa, hedefRowYeni)
                # # self.layoutChanged.emit()
                # self.endMoveRows()
            # print(sourceRow, hedefRow, hedefRowYeni)
            self.removeRow(sourceRow, sourceParent)
            # self.beginMoveRows(sourceParent, sourceRow, count, hedefParentIndex, hedefRowYeni)
            # kaynak_ust_sayfa.sayfa_sil(tasinan_sayfa)
            self.beginInsertRows(hedefParentIndex, hedefRowYeni, hedefRowYeni)
            hedef_ust_sayfa.sayfa_ekle(tasinan_sayfa)
            self.endInsertRows()

        # hedef row burda gercek
        else:  # hedef row itemin alt veya ustu farkli parent de olabilir
            hedefRowYeni = hedefRow
            if hedef_ust_sayfa == kaynak_ust_sayfa:
                # sourceRow < hedefRow  icin hedefRow dogru degerde geliyor
                if sourceRow > hedefRow:
                    hedefRowYeni = hedefRow + 1
            else:
                hedefRowYeni = hedefRow + 1
            self.removeRow(sourceRow, sourceParent)
            # kaynak_ust_sayfa.sayfa_sil(tasinan_sayfa)
            # self.beginMoveRows(sourceParent, sourceRow, count, hedefParentIndex, hedefRow)
            # print(sourceRow, hedefRow, hedefRowYeni)
            self.beginInsertRows(hedefParentIndex, hedefRowYeni, hedefRowYeni)
            hedef_ust_sayfa.sayfa_araya_ekle(hedefRowYeni, tasinan_sayfa)
            self.endInsertRows()

            # if hedef_ust_sayfa == kaynak_ust_sayfa:
            #     self.beginMoveRows(sourceParent, sourceRow, count, hedefParentIndex, hedefRowYeni)
            #     hedef_ust_sayfa.sayfa_sira_degistir(tasinan_sayfa, hedefRowYeni)
            #     self.layoutChanged.emit()
            #     self.endMoveRows()
            # else:

        # print("moveRows",sourceParent, sourceRow, count, destinationParent, destinationChildRow)
        # self.data = self.data[1] + self.data[0] + self.data[2]
        # self.endMoveRows()
        # print("moveRows finished")

    # # ---------------------------------------------------------------------
    # def moveRow(self, QModelIndex, p_int, QModelIndex_1, p_int_1):
    # sourceParent, sourceRow, destinationParent, destinationChild)
    #     print("asdasdsadasdasd")
    #     pass

    # ---------------------------------------------------------------------
    # bu 4 her modelde lazım
    # ---------------------------------------------------------------------
    def flags(self, index: QModelIndex):
        """her modelde mutlaka lazım,
        editable icin ayrica Qt::ItemIsEditable lazım"""
        # if not index.isValid():
        #     return 0

        return Qt.ItemFlag.ItemIsEditable | Qt.ItemFlag.ItemIsEnabled | \
               Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsDragEnabled | \
               Qt.ItemFlag.ItemIsDropEnabled
        # return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    # ---------------------------------------------------------------------
    def data(self, index: QModelIndex, role: int = None):
        """her modelde mutlaka lazım"""
        if not index.isValid():
            # return None
            return

        # TODO: bu alt satir problem, >= eşitten mi acaba, bazi satirlari bos gosteriyordu.
        # if index.row() >= self.rowCount():
        #     return QVariant()

        # if not index.parent().isValid():
        #     return None

        # if role != Qt.DisplayRole and role != Qt.EditRole:
        #     return None

        # her row her column icin cagriliyor bu,,
        # pixel pixel set etmek gibi eski tvlerde kayan nokta
        sayfa = self.sayfa_indexten(index)
        # print(sayfa.adi, "sayfa adi datadan")
        # sayfa = index.internalPointer()        # return item.data(index.column())

        # !!! delege kullandigimiz icin burda buyuk ihtimalle sadece qt.editrole kismi kullaniliyor.
        # o da cift tiklayinca mesela

        if role == Qt.ItemDataRole.DisplayRole:
            # return self.items[index.row()]
            return sayfa.adi

        if role == Qt.ItemDataRole.EditRole:
            if sayfa.adi[0] == "★":
                return sayfa.adi[2:]
            return sayfa.adi

        elif role == Qt.ItemDataRole.ToolTipRole:
            return sayfa.adi

        elif role == Qt.ItemDataRole.DecorationRole:
            return sayfa.ikon

        elif role == Qt.ItemDataRole.ForegroundRole:
            if sayfa.scene.isModified():
                return renk_degismis_nesne_yazi_rengi
            else:
                return self.yaziRengi
        elif role == Qt.ItemDataRole.SizeHintRole:
            return self.treeViewIkonBoyutu

        # elif role == Qt.ItemDataRole.BackgroundRole:
        #
        #     if self.items[index.row()][:2] == "!!":
        #         return QColor(50, 50, 50)
        #         # return self.colorPathAlternateA
        #     else:
        #         # if self.data(index, QtCore.Qt.ItemDataRole.ToolTipRole) in self.openPaths:
        #         if index.data(Qt.ItemDataRole.DisplayRole) == self.activePath:
        #             # return QtGui.QColor(100,100,255)
        #             return self.colorActivePathBg
        #         else:
        #             if index.row() % 2 == 0:
        #                 # return QtGui.QColor(100,100,100)
        #                 return self.colorPathAlternateA
        #             else:
        #                 # return QtGui.QColor(80,80,80)
        #                 return self.colorPathAlternateB

        # elif role == Qt.ItemDataRole.ForegroundRole:
        #
        #     if self.items[index.row()][:2] == "!!":
        #         return QColor(100, 100, 100)
        #     else:
        #         if index.data(Qt.ItemDataRole.DisplayRole) == self.activePath:
        #             # return QtGui.QColor(100,255,100)
        #             return self.colorActivePathText
        #         else:
        #             # return QtGui.QColor(200,200,200)
        #             return self.colorPathsText

        # elif role == Qt.ItemDataRole.FontRole:
        #     font = QFont()
        #     if index.data(Qt.ItemDataRole.DisplayRole) == self.activePath:
        #
        #         font.setBold(True)
        #     else:
        #         font.setBold(False)
        #
        #     if index.data(Qt.ItemDataRole.DisplayRole) in self.openPaths:
        #         font.setItalic(True)
        #     else:
        #         font.setItalic(False)
        #
        #     return font

        else:
            return  # None

    # ---------------------------------------------------------------------
    def headerData(self, section, orientation, role=None):
        """her modelde mutlaka lazım"""
        # if orientation == Qt.Horizontal and role == Qt.DisplayRole:
        if (orientation, role) == (Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole):
            # return self.root.data(section)
            return "Sayfalar"
        return "Sayfalar"

    # ---------------------------------------------------------------------
    # def rowCount(self, parentIndex=QModelIndex()):
    def rowCount(self, parent=QModelIndex()):
        """her modelde mutlaka lazım"""
        # return self.kokSayfa.satirSayısı()
        #
        # parentItem = self.getItem(parent)
        #
        # return parentItem.childCount()

        # parent = self.index_sayfadan(parentIndex)

        # if parent.column() > 0:
        #     return 0

        if not parent.isValid():
            parentNode = self.kokSayfa
        else:
            parentNode = parent.internalPointer()

        return parentNode.satirSayisi()
        # return len(parentNode)

    # ---------------------------------------------------------------------
    # tree, table icin
    # ---------------------------------------------------------------------
    def columnCount(self, parent: QModelIndex = None, *args, **kwargs) -> int:
        """ the following functions must be implemented in direct subclasses of
         QAbstractTableModel and QAbstractItemModel:"""

        # return self.kokSayfa.sutunSayısı()

        return 1

        # if index.isValid():
        #     return index.internalPointer().columnCount()
        # return self._root.columnCount()

    # ---------------------------------------------------------------------
    def setData(self, index: QModelIndex, value: Any, role=Qt.ItemDataRole.DisplayRole):
        if not index.isValid():
            return False

        sayfa = self.sayfa_indexten(index)
        # if role == Qt.ItemDataRole.DisplayRole:
        #     sayfa.adi = value
        #     # print(value)
        #     self.dataChanged.emit(index, index)
        #     return True

        if role == Qt.ItemDataRole.EditRole:
            # self.items[index.row()] = value
            if sayfa.adi == value:  # cift tiklayip mesela hic bir sey degistirmeyince degisimis diye isaretlenmesin..
                return False

            sayfa_eski_adi = sayfa.adi
            if sayfa.scene.isModified():
                sayfa.adi = "★ {}".format(value)
            else:
                sayfa.adi = value

            self.dataChanged.emit(index, index)
            self.nesneDegisti.emit(self.sayfa_indexten(index), sayfa_eski_adi)
            return True

        return False

    # ---------------------------------------------------------------------
    def setHeaderData(self, p_int, Qt_Orientation, _any, role=None):
        self.headerDataChanged.emit()

    # ---------------------------------------------------------------------
    # resizable
    # ---------------------------------------------------------------------

    # ---------------------------------------------------------------------
    def sayfaEkleIndexIle(self, sayfa, index):
        if not index or not index.isValid():
            ustSayfa = self.kokSayfa
        else:
            ustSayfa = index.internalPointer()
            ustSayfa.sayfa_ekle(sayfa)

    # ---------------------------------------------------------------------
    def insertRows(self, position, rows, parent: QModelIndex = None, sayfa=None, *args, **kwargs):
        # print("insert")
        parentSayfa = self.sayfa_indexten(parent)
        self.beginInsertRows(parent, position, position + rows - 1)

        sonuc = False
        for row in range(rows):
            sayfa = Sayfa(adi='Unknown')
            sonuc = parentSayfa.sayfa_araya_ekle(row, sayfa)
        self.endInsertRows()
        return sonuc

    # ---------------------------------------------------------------------
    def sayfa_ekle(self, satir=None, scene=None, view=None, ikon=None, ustSayfa=None):

        adi = self.tr("Untitled")
        sayfa = Sayfa(adi=adi, scene=scene, view=view, ikon=ikon, parent=ustSayfa)

        # parentIndex = self.index_sayfadan(ustSayfa)

        # self.beginInsertRows(QModelIndex(), position, position + rows - 1)
        # pos =ustSayfa.sayfa_sayisi()
        # self.beginInsertRows(QModelIndex(),pos ,pos)

        if satir:
            if ustSayfa == self.kokSayfa:
                # print("aaa")
                self.kokSayfa.sayfa_araya_ekle(satir + 1, sayfa)
            else:
                # print("bbbb")

                ustSayfa.sayfa_araya_ekle(satir + 1, sayfa)
        else:  # alt sayfa eklerken
            # if ustSayfa == self.kokSayfa:
            #     self.kokSayfa.sayfa_ekle(sayfa)
            # else:
            #     ustSayfa.sayfa_ekle(sayfa)

            if ustSayfa == self.kokSayfa:
                # print("cccc")
                self.kokSayfa.sayfa_ekle(sayfa)

            else:
                # print("ddd")
                # print(ustSayfa.ic_sayfalar())
                ustSayfa.sayfa_ekle(sayfa)
                # print(ustSayfa.ic_sayfalar())

        # self.endInsertRows()

        # idx = self.index_sayfadan(sayfa)
        # self.dataChanged.emit(parentIndex, parentIndex)
        # self.dataChanged.emit(idx, idx)
        if not ustSayfa:
            self.enSonAktifSayfa = sayfa
        self.layoutChanged.emit()

        # print("----" * 5)
        #
        # for ic_sayfa in self.kokSayfa.ic_sayfalar():
        #     print(ic_sayfa.adi)
        #     for ic_sayfa2 in ic_sayfa.ic_sayfalar():
        #         print("--    ",ic_sayfa2.adi)
        #         for ic_sayfa3 in ic_sayfa2.ic_sayfalar():
        #             print("++       ", ic_sayfa3.adi)
        #             for ic_sayfa4 in ic_sayfa3.ic_sayfalar():
        #                 print("**           ", ic_sayfa4.adi)
        #
        # # print(list(self.iterItems(self.kokSayfa)))
        # print("----" * 5)

        return sayfa

    # ---------------------------------------------------------------------
    def sifirla(self):
        # TODO: eski sayfa cop toplanıyor mu
        self.beginResetModel()
        self.kokSayfa = Sayfa()
        self.endResetModel()

    # ---------------------------------------------------------------------
    def satir_sil(self, sayfa):
        row = sayfa.satir()
        # scene = sayfa.scene
        # view = sayfa.view

        # if sayfa._ic_sayfalar:
        #     self._ic_sayfa_sil(sayfa._ic_sayfalar)

        # TODO:: bu  alttaki tek satir da hallediyor eger tek row silinecekse
        # self.removeRow(row, self.index_sayfadan(sayfa.parent()))

        # if not sayfa.parent() is self.kokSayfa:

        if not sayfa.parent() == self.kokSayfa:
            # print("sil, parent var")
            self.beginRemoveRows(self.index_sayfadan(sayfa.parent()), row, row)
            # itemPointerList = sayfa.parent().takeRow(row)
            # sonuc = self.removeRow(row, self.index_sayfadan(sayfa.parent()))
            sonuc = sayfa.parent().sayfa_sil(sayfa)
        else:
            self.beginRemoveRows(QModelIndex(), row, row)
            # sonuc = self.removeRow(row, QModelIndex())
            sonuc = self.kokSayfa.sayfa_sil(sayfa)

        self.endRemoveRows()
        # self.layoutChanged.emit()

        # print(sonuc, "removed")

        # if sonuc:
        #     ##################################
        #     # !!! ONEMLI ONEMLI ONEMLI !!!
        #     ## bu alttaki  grup yukarda iken bir tabtan sayfa silince diger tabtaki dokumani kapatabiliyordu veya
        #     ## sayfa silinen dokumani da kapatabiliyordu. (kapatmak = tabi silmek), o yuzden asagi aldik,
        #     ## modelden sayfa silindikten sonra bu alttaki temizlikleri yapiyoruz artik..
        #     ## muhtemelen asagida view set parent none diyoruz, o esnada tabwidgetin o anki widgeti view oldugu icin
        #     ## tabwidget viewi kaybedince yerine de yenisi gelmeyince,ki
        #     ## tabwidget updatelerini kapamamaiza ragmen boyleydi..
        #     ## ...
        #     ## simdi model degistigi icin tabWidget in o anki widgeti da yeni model gore set ediliyor
        #     ## dolayısı ile alttakini silebiliyoruz.
        #     # del sayfa
        #     # scene.parent(None) yapmak, scene.undoStack lari temizlemiyor..
        #     # sayfa.scene.parent().undoGroup.blockSignals(True)
        #     scene.parent().undoGroup.removeStack(scene.undoStack)
        #     scene.undoStack.setParent(None)
        #     scene.undoStack.deleteLater()
        #     # sayfa.scene.parent().undoGroup.blockSignals(False)
        #
        #     view.setParent(None)
        #     scene.setParent(None)
        #     view.deleteLater()
        #     scene.deleteLater()

    # ---------------------------------------------------------------------
    def removeRows(self, row, count, parent: QModelIndex = None, *args, **kwargs):
        if row < 0 or row > self.rowCount():
            return False
        self.beginRemoveRows(parent, row, row + count - 1)

        parentSayfa = self.sayfa_indexten(parent)
        # sayfa = parentSayfa.ic_sayfa(row)

        # parentSayfa.sayfa_sil(sayfa)
        # print(parentSayfa)
        # print(self.kokSayfa)
        # print(parentSayfa == self.kokSayfa, "esitler imisler")
        sonuc = parentSayfa.sayfa_sil_sira_ile(row)

        self.endRemoveRows()
        self.layoutChanged.emit()
        return sonuc

        # node = self.sayfa_indexten(parent)
        # self.beginRemoveRows(parent, position,
        #                      position + count - 1)
        # for row in range(count):
        #     del node.children[position + row]
        # self.endRemoveRows()
        # return True

    # def insertColumns(self, p_int, parent=None, *args, **kwargs):
    #     self.beginInsertColumns()
    #     self.endInsertColumns()
    #
    # def removeColumns(self, p_int, p_int_1, parent=None, *args, **kwargs):
    #     self.beginRemoveColumns()
    #     self.endRemoveColumns()

    # ---------------------------------------------------------------------
    # diger
    # ---------------------------------------------------------------------



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtCore import Qt, QRectF, QSizeF, QPointF, Slot, QBuffer, QIODevice
from PySide6.QtGui import QPainterPath, QPainterPathStroker, QPen, QBrush, QColor, QTransform, QTextOption, QPainter
from PySide6.QtSvg import QSvgGenerator
from PySide6.QtWidgets import QGraphicsItem, QStyle
from .. import shared
from ..nesneler.tempTextItem import TempTextItem


########################################################################
class PathItem(QGraphicsItem):
    Type = shared.PATH_ITEM_TYPE

    def __init__(self, pos, yaziRengi, arkaPlanRengi, pen, font, path=None, parent=None):
        super(PathItem, self).__init__(parent)

        self._kim = shared.kim(kac_basamak=16)
        self.setAcceptHoverEvents(True)

        self.setPos(pos)

        self._rect = QRectF()

        if path:
            self._path = path
        else:
            self._path = QPainterPath()
            # self.append(pos)  # buna gerek yok..

        # self.pixmap = QPixmap()
        self.secilen_nokta = None

        self.movedPointsList = []

        # cizim icin False baslamak dogru olabilirdi ama
        # kopyalama ve dosyadan yukleme gibi durumlarda bu false kalıyor
        # o yuzden True baslayip, cizim de ilk noktada(move_start_point) False ediyoruz
        self.isDrawingFinished = True
        self.realCount = 0
        self.lastPath = QPainterPath()

        self.startPointRect = QRectF()
        self.searchBoxSize = QSizeF(15, 15)
        self.intersects = False

        self.isPathClosed = False

        self.cizgi_basitlestirme_miktari = 10  # moveCenterda kullanildigi icin yarisi aslinda

        self.initialFlags = self.flags()
        self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)

        self.secili_nesne_kalem_kalinligi = 0

        self._pen = pen
        self._brush = QBrush()
        self._font = font

        self.activeItemLineColor = shared.activeItemLineColor
        self.cizgiRengi = pen.color()
        self.setCizgiRengi(self.cizgiRengi)  # also sets self._pen
        self.yaziRengi = yaziRengi
        self.setYaziRengi(yaziRengi)
        self.setArkaPlanRengi(arkaPlanRengi)  # also sets self._brush

        # we override bg brush for drawing.
        self.setBrush(Qt.BrushStyle.NoBrush)
        # self.setBrush(arkaPlanRengi)

        self.cosmeticSelect = False
        self.isActiveItem = False
        self._isPinned = False
        self.ustGrup = None

        self._text = ""
        self._command = {}

        self.editMode = False

        self.tempTextItem = None
        self.tempEskiText = ""

        self.textPadding = 20

        self.painterTextOption = QTextOption()
        self.painterTextOption.setWrapMode(QTextOption.WrapMode.WrapAtWordBoundaryOrAnywhere)
        self.painterTextOption.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.oklar_dxdy_nokta = {}

        self._resizing = False
        self.handleSize = 10
        self.resizeHandleSize = QSizeF(self.handleSize, self.handleSize)
        self.boundingRectTasmaDegeri = self.handleSize / 2

        self.painterTextRect = QRectF(self.boundingRect())

        self.update_resize_handles()

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return
        # self.prepareGeometryChange()
        self._rect = rect
        self.update_resize_handles()
        # self.update()

    # ---------------------------------------------------------------------
    def rect(self):
        return self._rect

    # ---------------------------------------------------------------------
    def update_resize_handles(self):
        self.prepareGeometryChange()

        if self.scene():
            resizeHandleSize = self.resizeHandleSize / self.scene().views()[0].transform().m11()
        else:
            resizeHandleSize = self.resizeHandleSize

        self.topLeftHandle = QRectF(self._rect.topLeft(), resizeHandleSize)
        self.topRightHandle = QRectF(self._rect.topRight(), resizeHandleSize)
        self.bottomRightHandle = QRectF(self._rect.bottomRight(), resizeHandleSize)
        self.bottomLeftHandle = QRectF(self._rect.bottomLeft(), resizeHandleSize)

        self.topLeftHandle.moveCenter(self._rect.topLeft())
        self.topRightHandle.moveCenter(self._rect.topRight())
        self.bottomRightHandle.moveCenter(self._rect.bottomRight())
        self.bottomLeftHandle.moveCenter(self._rect.bottomLeft())

    # ---------------------------------------------------------------------
    def hide_resize_handles(self):
        self.isPinned = True

    # ---------------------------------------------------------------------
    def show_resize_handles(self):
        self.isPinned = False

    # ---------------------------------------------------------------------
    def type(self):
        return PathItem.Type

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      # "painterPath": self._path,
                      "kim": self._kim,
                      "painterPathAsList": self.toList(),
                      "isPathClosed": self.isPathClosed,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "pen": self._pen,
                      "font": self._font,
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "yaziHiza": int(self.painterTextOption.alignment()),
                      "text": self.text(),
                      "isPinned": self.isPinned,
                      "command": self._command,
                      }
        return properties

    # ---------------------------------------------------------------------
    @property
    def isPinned(self):
        return self._isPinned

    # ---------------------------------------------------------------------
    @isPinned.setter
    def isPinned(self, value):
        if value:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          # | QGraphicsItem.GraphicsItemFlag.ItemIsMovable
                          #  | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable
                          )

        else:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsMovable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
        self._isPinned = value

    # ---------------------------------------------------------------------
    def ok_ekle(self, ok, scenepPos, okunHangiNoktasi):

        # self.oklar_dxdy_nokta.append((ok, self.mapFromScene(scenePos)))
        sceneCenter = self.sceneCenter()
        dx = sceneCenter.x() - scenepPos.x()
        dy = sceneCenter.y() - scenepPos.y()

        # self.oklar_dxdy_nokta[ok._kim] = (dx, dy, nokta)
        self.oklar_dxdy_nokta[ok] = (dx, dy, okunHangiNoktasi)

        ok.baglanmis_nesneler[self._kim] = okunHangiNoktasi

    # ---------------------------------------------------------------------
    def ok_sil(self, ok):
        del self.oklar_dxdy_nokta[ok]
        del ok.baglanmis_nesneler[self._kim]

    # ---------------------------------------------------------------------
    def ok_guncelle(self):
        if self.oklar_dxdy_nokta:
            sceneCenter = self.sceneCenter()
            scx = sceneCenter.x()
            scy = sceneCenter.y()
            for ok, dxdy_nokta in self.oklar_dxdy_nokta.items():
                if dxdy_nokta[2] == 1:
                    ok.temp_prepend(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                elif dxdy_nokta[2] == 2:
                    ok.temp_append(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                if ok._text:
                    ok.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None

    # # ---------------------------------------------------------------------
    # def isPathClosed(self):
    #     poly = self._path.toSubpathPolygons()
    #     return poly[0].isClosed()

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):
        # if change == self.ItemPositionChange:
        # if change == self.ItemSelectedChange:
        # if change == self.ItemSelectedHasChanged:
        if change == QGraphicsItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().path_item_selected(self)
            else:
                self.scene().parent().item_deselected(self)

        # return QGraphicsRectItem.itemChange(self, change, value)
        return value
        # super(Rect, self).itemChange(change, value)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base - text - path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_nesne_sag_menu_about_to_show(self)
        self.scene().parent().nesneSagMenu.popup(event.screenPos())

        event.accept()

    # ---------------------------------------------------------------------
    def sceneCenter(self):
        if self.parentItem():
            return self.mapToParent(self._path.controlPointRect().center())
        return self.mapToScene(self._path.controlPointRect().center())

    # ---------------------------------------------------------------------
    def sceneWidth(self):
        return self.sceneRight() - self.sceneLeft()

    # ---------------------------------------------------------------------
    def sceneHeight(self):
        return self.sceneBottom() - self.sceneTop()

    # ---------------------------------------------------------------------
    def sceneRight(self):

        path = self.mapToScene(self._path)
        liste = self.toList(path)
        right = liste[0]
        for _ in range(path.elementCount()):
            right = max(liste, key=lambda e: e[0])

        return right[0]

    # ---------------------------------------------------------------------
    def sceneLeft(self):

        path = self.mapToScene(self._path)
        liste = self.toList(path)
        left = liste[0]
        for _ in range(path.elementCount()):
            left = min(liste, key=lambda e: e[0])

        return left[0]

    # ---------------------------------------------------------------------
    def sceneTop(self):
        path = self.mapToScene(self._path)
        liste = self.toList(path)
        top = liste[1]
        for _ in range(path.elementCount()):
            top = min(liste, key=lambda e: e[1])

        return top[1]

    # ---------------------------------------------------------------------
    def sceneBottom(self):
        path = self.mapToScene(self._path)
        liste = self.toList(path)
        bottom = liste[1]
        for _ in range(path.elementCount()):
            bottom = max(liste, key=lambda e: e[1])

        return bottom[1]

    # ---------------------------------------------------------------------
    def setSceneLeft(self, left):
        self.setX(left + self.scenePos().x() - self.sceneLeft())

    # ---------------------------------------------------------------------
    def setSceneRight(self, right):
        self.setX(right + self.scenePos().x() - self.sceneRight())

    # ---------------------------------------------------------------------
    def setSceneTop(self, top):
        self.setY(top + self.scenePos().y() - self.sceneTop())

    # ---------------------------------------------------------------------
    def setSceneBottom(self, bottom):
        self.setY(bottom + self.scenePos().y() - self.sceneBottom())

    # ---------------------------------------------------------------------
    def setHorizontalCenter(self, hcenter):
        self.setY(hcenter + self.scenePos().y() - self.sceneCenter().y())

    # ---------------------------------------------------------------------
    def setVerticalCenter(self, vcenter):
        self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x())

    # ---------------------------------------------------------------------
    def path(self):
        return self._path

    # ---------------------------------------------------------------------
    def setPath(self, path):
        if self._path == path:
            return
        self.prepareGeometryChange()
        self._path = path
        self._boundingRect = QRectF()
        self.update()

    # ---------------------------------------------------------------------
    def toList(self, path=None):
        if not path:
            path = self._path
        liste = []
        for i in range(path.elementCount()):
            element = path.elementAt(i)
            liste.append((element.x, element.y))
        return liste

    # ---------------------------------------------------------------------
    def fromList(self, pathElementsList, closePath=False):
        path = QPainterPath()
        path.moveTo(*pathElementsList[0])
        for e in pathElementsList[1:]:
            path.lineTo(*e)
        if closePath:
            path.closeSubpath()
            self.isPathClosed = True
            self.setBrush(self.arkaPlanRengi)
        self.setPath(path)

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):
        _selectionLineBgColor = QColor.fromHsv(col.hue(),
                                               0,
                                               20 if col.value() > 127 else 250)

        _activeItemLineColor = QColor(self.activeItemLineColor)
        if col.hue() > 300 or col.hue() < 30:
            _activeItemLineColor.setHsv((col.hue() + 150) % 360,
                                        self.activeItemLineColor.saturation(),
                                        self.activeItemLineColor.value())
        self._pen.setColor(col)
        self.cizgiRengi = col

        self.selectionPenBottom = QPen(_selectionLineBgColor,
                                       self.secili_nesne_kalem_kalinligi,
                                       Qt.PenStyle.DashLine,
                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)
        self.selectionPenBottomIfAlsoActiveItem = QPen(_activeItemLineColor,
                                                       self.secili_nesne_kalem_kalinligi,
                                                       Qt.PenStyle.DashLine,
                                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)

        if col.value() > 245:
            self.handleBrush = shared.handleBrushKoyu
        else:
            self.handleBrush = shared.handleBrushAcik
        self.update()

    # ---------------------------------------------------------------------
    def setYaziRengi(self, col):
        self.yaziRengi = col
        self.update()

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.boundingRectTasmaDegeri = self.handleSize / 2
        self.update()

    # ---------------------------------------------------------------------
    def update(self, rect=QRectF()):
        # self.pathi_pixmap_yap()
        super(PathItem, self).update(rect)

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        self.arkaPlanRengi = col
        # burda nesne kapali mi degil mi kontrolu yapiyoruz
        # daha iyi olabilir bu kontrol
        if not self._brush == Qt.BrushStyle.NoBrush:
            self.setBrush(QBrush(col))

    # ---------------------------------------------------------------------
    def setText(self, text):
        self._text = text
        # self.update()
        self.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def text(self):
        return self._text

    # ---------------------------------------------------------------------
    def setCommand(self, title, command):
        self._command["title"] = title
        self._command["command"] = command

    # ---------------------------------------------------------------------
    def command(self):
        return self._command

    # ---------------------------------------------------------------------
    def setFont(self, font):
        font.setBold(self._font.bold())
        font.setItalic(self._font.italic())
        font.setUnderline(self._font.underline())
        font.setStrikeOut(self._font.strikeOut())
        font.setOverline(self._font.overline())
        font.setPointSizeF(self._font.pointSizeF())
        self._font = font
        self.update()

    # ---------------------------------------------------------------------
    def font(self):
        return self._font

    # ---------------------------------------------------------------------
    def setFontPointSizeF(self, fontPointSizeF):
        self._font.setPointSizeF(fontPointSizeF)
        self.update()

    # ---------------------------------------------------------------------
    def fontPointSizeF(self):
        return self._font.pointSizeF()

    # ---------------------------------------------------------------------
    def ver_yazi_hizasi(self):
        return self.painterTextOption.alignment()

    # ---------------------------------------------------------------------
    def kur_yazi_hizasi(self, hizalama):
        self.painterTextOption.setAlignment(hizalama | Qt.AlignmentFlag.AlignVCenter)
        # self.painterTextOption.setAlignment(hizalama)
        self.update()

    # ---------------------------------------------------------------------
    def ver_karakter_bicimi(self):
        # return self._font.bold | self._font.italic | self._font.under | self._font.strikeout
        return {"b": self._font.bold(),
                "i": self._font.italic(),
                "u": self._font.underline(),
                "s": self._font.strikeOut(),
                "o": self._font.overline(),
                }

    # ---------------------------------------------------------------------
    def kur_karakter_bicimi(self, sozluk):
        self._font.setBold(sozluk["b"])
        self._font.setItalic(sozluk["i"])
        self._font.setUnderline(sozluk["u"])
        self._font.setStrikeOut(sozluk["s"])
        self._font.setOverline(sozluk["o"])
        self.update()

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def setBrush(self, brush):

        if self._brush == brush:
            return
        self._brush = brush
        self.update()

    # ---------------------------------------------------------------------
    def brush(self):
        return self._brush

    # ---------------------------------------------------------------------
    def mirror(self, x, y):
        if not x and not y:
            return
        # mirrorMatrix = QMatrix()
        # PyQt5 de matrix yok, onun yerine qtransform..
        mirrorMatrix = QTransform()
        if x:
            if y:
                mirrorMatrix.scale(-1, -1)
            else:
                mirrorMatrix.scale(-1, 1)
        else:
            if y:
                mirrorMatrix.scale(1, -1)

        mirroredPath = self._path * mirrorMatrix
        # mirroredPath = mirrorMatrix.map(self._path)
        self.setPath(mirroredPath)

    # ---------------------------------------------------------------------
    def flipHorizontal(self, mposx):
        self.mirror(x=True, y=False)
        ipos = self.scenePos()
        # ipos.setX(ipos.x() + 2 * (mposx - ipos.x()))
        ipos.setX((2 * mposx) - ipos.x())
        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            ipos = (self.parentItem().mapFromScene(ipos))

        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.setRotation(0)
        for c in self.childItems():
            c.flipHorizontal(self.sceneCenter().x())
        if eskiRot:
            self.setRotation(eskiRot)

    # ---------------------------------------------------------------------
    def flipVertical(self, mposy):
        self.mirror(x=False, y=True)
        ipos = self.scenePos()
        # ipos.setX(ipos.x() + 2 * (mposx - ipos.x()))
        ipos.setY((2 * mposy) - ipos.y())
        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            ipos = (self.parentItem().mapFromScene(ipos))

        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.setRotation(0)
        for c in self.childItems():
            c.flipVertical(self.sceneCenter().y())
        if eskiRot:
            self.setRotation(eskiRot)

    # ---------------------------------------------------------------------
    def move_start_point(self):
        # point = self.mapFromScene(point)
        self.isDrawingFinished = False
        point = QPointF(0, 0)
        path = QPainterPath(self._path)
        if not path.elementCount():
            path.moveTo(point)
            self.realCount = path.elementCount()
            self.startPointRect = QRectF(point, self.searchBoxSize)
            self.startPointRect.moveCenter(point)
            self.setPath(path)
            self.setFlags(self.initialFlags)

    # ---------------------------------------------------------------------
    def append(self, point):
        point = self.mapFromScene(point)
        path = QPainterPath(self._path)
        path.lineTo(point)
        self.lastPath = QPainterPath(path)
        self.setPath(path)
        self.realCount = path.elementCount()

    # ---------------------------------------------------------------------
    def temp_append_and_replace_last(self, point):
        point = self.mapFromScene(point)
        path = QPainterPath(self._path)
        # print(path.elementCount(), self.realCount)
        # if path.elementCount() == 0:
        #     path.lineTo(point)
        #
        # elif path.elementCount() == self.realCount:
        #     path.lineTo(point)
        #
        # elif path.elementCount() < self.realCount:
        #     path.lineTo(point)

        # realcount 0 basliyor zaten <= esiti 0 esitligini sagliyor, ileriye referans olabilir usttekiler kalsin
        if path.elementCount() <= self.realCount:
            path.lineTo(point)

        # bu replace last kismi, else icinde cunku mouse sabitken birden fazla nokta silinirse,
        # cizili kalan son noktayi,  temp nokta  gibi algilayip bir nokta daha eksiltmis oluyordu.
        else:
            path.setElementPositionAt(path.elementCount() - 1, point.x(), point.y())

        self.setPath(path)

    # ---------------------------------------------------------------------
    def sonNoktaSil(self):
        # path = QPainterPath(self._path)
        nokta_listesi = self.toList()
        self.fromList(nokta_listesi[:-1], closePath=False)
        self.realCount = len(nokta_listesi) - 1

    # ---------------------------------------------------------------------
    def check_if_at_start(self, pos):
        pos = self.mapFromScene(pos)
        if self.realCount > 1:
            currentPosRect = QRectF(pos, self.searchBoxSize)
            currentPosRect.moveCenter(pos)

            if currentPosRect.intersects(self.startPointRect):
                self.intersects = True
            else:
                self.intersects = False
        else:
            self.intersects = False

    # ---------------------------------------------------------------------
    def close_path(self, kapat):
        if kapat:
            self.lastPath.closeSubpath()
            self.isPathClosed = True
            self.setBrush(self.arkaPlanRengi)
        self.basitlestir()
        # basitlestir() -> self.setPath yapiyor
        # self.setPath(self.lastPath)
        self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
        self.intersects = False
        # self.setSelected(True)

        # update pos - bu onemli
        crect = self._path.controlPointRect()
        crect = self.mapRectToScene(crect)
        # cunku pos noktasi merkezde
        diff = self.scenePos() - crect.center()
        self.setPos(crect.center())
        self._path.translate(diff)
        self.update_painter_text_rect()

        self.isDrawingFinished = True

        # self.pathi_pixmap_yap()

        # print(self.isPathClosed())
        self.scene().unite_with_scene_rect(self.sceneBoundingRect())

    # # ---------------------------------------------------------------------
    # def basitlestir_1(self):
    #     liste = []
    #     try:
    #         for i in range(self.lastPath.elementCount()):
    #             # a = self.lastPath.elementAt(i).x + self._path.elementAt(i).y
    #             # b = self.lastPath.elementAt(i+1).x + self._path.elementAt(i+1).y
    #             # if abs(a-b) > 2:
    #
    #             # a = abs(self.lastPath.elementAt(i).x - self._path.elementAt(i+1).x)
    #             # b = abs(self.lastPath.elementAt(i).y - self._path.elementAt(i + 1).y)
    #             # if a + b > 2:
    #             #     liste.append([self.lastPath.elementAt(i).x, self.lastPath.elementAt(i).y])
    #
    #             a = abs(self.lastPath.elementAt(i).x - self._path.elementAt(i + 1).x)
    #             if a < 2:
    #                 b = abs(self.lastPath.elementAt(i).y - self._path.elementAt(i + 1).y)
    #                 if b < 2:
    #                     liste.append([self.lastPath.elementAt(i).x, self.lastPath.elementAt(i).y])
    #                 else:
    #                     i += 1
    #
    #     except Exception as e:
    #         pass
    #     print(self.lastPath.elementCount(), len(liste))
    #     self.fromList(liste, self.isPathClosed())

    # ---------------------------------------------------------------------
    def basitlestir(self):
        if self.lastPath.elementCount() > 3:
            liste = []
            # ilk noktayi basitlestirilmis cizgiye dahil ediyoruz
            liste.append([self.lastPath.elementAt(1).x, self.lastPath.elementAt(1).y])
            self.arama_kutusu = QRectF(0, 0, self.cizgi_basitlestirme_miktari, self.cizgi_basitlestirme_miktari)
            self.arama_kutusu.moveCenter(QPointF(liste[0][0], liste[0][1]))
            try:
                for i in range(self.lastPath.elementCount()):
                    if self.arama_kutusu.contains(self.lastPath.elementAt(i).x, self.lastPath.elementAt(i).y):
                        continue
                    else:
                        liste.append([self.lastPath.elementAt(i).x, self.lastPath.elementAt(i).y])
                        self.arama_kutusu.moveCenter(QPointF(liste[-1][0], liste[-1][1]))

                # son noktayi basitlestirilmis cizgiye dahil ediyoruz
                liste.append([self.lastPath.elementAt(self.lastPath.elementCount() - 1).x,
                              self.lastPath.elementAt(self.lastPath.elementCount() - 1).y])
            #
            except Exception as e:
                print(e)
            # print(self.lastPath.elementCount(), len(liste))
            self.fromList(liste, self.isPathClosed)

    # # ---------------------------------------------------------------------
    # def pathi_pixmap_yap(self):
    #     # self.pixmap.setSize(self.boundingRect().size())
    #     # TODO bunu iptal edebiliriz, performans sagliyor mu acaba pixmape cevirmek
    #     # > 1 idi, gecici iptal ve test icin 10000 yaptik
    #     # TODO:2 saglamiyor gozukuyor. cok daha yavas, belli bir nokta sayisi uzerinde de yavaş
    #     if self._path.elementCount() > 50000:
    #         # TODO burda sizeF te size donusturmede ve de asaigdaki -self.boundingRect().topLeft() kismida
    #         #  kusurat kaybindan ufak kaymalar var
    #         pix = QPixmap(self.boundingRect().size().toSize())
    #         pix.fill(Qt.transparent)
    #         painter = QPainter(pix)
    #         # painter.setRenderHint(QPainter.Antialiasing)
    #         painter.setPen(self._pen)
    #         painter.setBrush(self._brush)
    #         # if not painter.begin(self.pixmap):
    #         #     print("hata")
    #         painter.setRenderHint(QPainter.Antialiasing)
    #         # painter.scale(.3, .3)
    #         painter.translate(-self.boundingRect().topLeft())
    #         painter.drawPath(self._path)
    #         self.pixmap = pix
    #         painter.end()
    #         painter = None
    #         del painter

    # ---------------------------------------------------------------------
    def qt_graphicsItem_shapeFromPath(self, path, pen):

        # We unfortunately need this hack as QPainterPathStroker will set a width of 1.0
        # if we pass a value of 0.0 to QPainterPathStroker.setWidth()
        penWidthZero = 0.00000001
        if path == QPainterPath():
            return path
        ps = QPainterPathStroker()
        ps.setCapStyle(pen.capStyle())

        if pen.widthF() <= 0:
            ps.setWidth(penWidthZero)
        else:
            ps.setWidth(pen.widthF())
        ps.setJoinStyle(pen.joinStyle())
        ps.setMiterLimit(pen.miterLimit())
        # to include connection indicator circle
        # path.addRect(self.startPointRect)
        path.addEllipse(self.startPointRect)
        p = ps.createStroke(path)  # returns QPainterPath
        p.addPath(path)
        return p

    # ---------------------------------------------------------------------
    def boundingRect(self):
        r = self.shape().controlPointRect()
        return r

        # return self._path.controlPointRect()

        # if self._boundingRect.isNull():
        #     if self.pen().widthF() == 0.0:
        #         self._boundingRect = self._path.controlPointRect()
        #     else:
        #         self._boundingRect = self.shape().controlPointRect()
        #
        # return self._boundingRect

    # ---------------------------------------------------------------------
    def shape(self):
        p = self.qt_graphicsItem_shapeFromPath(QPainterPath(self._path), self.pen())
        r = p.controlPointRect()
        if not self._resizing:
            self.setRect(QRectF(r))
        r.adjust(-self.boundingRectTasmaDegeri, -self.boundingRectTasmaDegeri,
                 self.boundingRectTasmaDegeri, self.boundingRectTasmaDegeri)
        p.addRect(r)
        return p

    # ---------------------------------------------------------------------
    def update_painter_text_rect(self):
        r = QRectF(self.boundingRect())
        size = self.boundingRect().size()
        size.setWidth(self.sceneWidth() - self.textPadding)
        r.moveCenter(self.boundingRect().center())
        self.painterTextRect = r

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(PathItem, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):

        # cursor = self.scene().parent().cursor()

        # if self.isSelected() and self.scene().aktifArac == self.scene().SecimAraci:
        if not self.ustGrup:
            if self.scene().aktifArac == self.scene().SecimAraci:
                if self.topLeftHandle.contains(event.pos()) or self.bottomRightHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeFDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeFDiagCursor, gecici_mi=True)
                elif self.topRightHandle.contains(event.pos()) or self.bottomLeftHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeBDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeBDiagCursor, gecici_mi=True)
                else:
                    self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(PathItem, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        # if self.isSelected():
        # self.setCursor(self.saved_cursor)
        if not self.ustGrup:
            self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(PathItem, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # factor = 1.41 ** (event.delta() / 240.0)

        if self.ustGrup:
            return QGraphicsItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & Qt.ControlModifier:
        if toplam == ctrlShift:
            self.scaleItemByScalingPath(event.delta())

        # elif event.modifiers() & Qt.ShiftModifier:
        elif toplam == shift:
            self.changeFontSizeF(event.delta())

        # elif event.modifiers() & Qt.AltModifier:
        elif toplam == alt:
            self.changeBackgroundColorAlpha(event.delta())

        elif toplam == ctrlAlt:
            self.changeTextColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())

        elif toplam == altShift:
            # self.changeImageItemTextBackgroundColorAlpha(event.delta())
            self.changeLineColorAlpha(event.delta())

        # elif toplam == ctrlAltShift:
        #     self.scaleItemByScalingPath(event.delta())

        if not self.isDrawingFinished:
            self.scene().tekerlek_ile_firca_boyu_degistir(event.delta())

        else:
            super(PathItem, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def changeFontSizeF(self, delta):

        if self.tempTextItem:
            return

        # font = self.font()
        size = self.fontPointSizeF()

        if delta > 0:
            # font.setPointSizeF(size + 1)
            size += 1

        else:
            if size > 10:
                # font.setPointSizeF(size - 1)
                size -= 1
            else:
                # undolari biriktermesin diye donuyoruz,
                # yoksa zaten ayni size de yeni bir undolu size komutu veriyor.
                return

        # self.setFont(font)
        if self.childItems():
            self.scene().undoStack.beginMacro("change text size")
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)
            for c in self.childItems():
                c.changeFontSizeF(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)

    # ---------------------------------------------------------------------
    def scaleItemByScalingPath(self, delta):

        if self.tempTextItem:
            return

        scaleFactor = 1.1
        if delta < 0:
            scaleFactor = 1 / scaleFactor

        scaleMatrix = QTransform()
        scaleMatrix.scale(scaleFactor, scaleFactor)
        scaledPath = self._path * scaleMatrix
        # self.setPath(scaledPath)
        self.scene().undoRedo.undoableScalePathItemByScalingPath(self.scene().undoStack, "scale", self,
                                                                 scaledPath, scaleFactor)

        # self.scene().unite_with_scene_rect(self.sceneBoundingRect())

        # self.setTransformOriginPoint(self.boundingRect().center())

    # ---------------------------------------------------------------------
    def rotateWithOffset(self, angle):
        cEski = self.sceneCenter()
        self.setRotation(angle)
        cYeni = self.sceneCenter()
        diff = cEski - cYeni
        self.moveBy(diff.x(), diff.y())
        self.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def rotateItem(self, delta):
        if not self.isDrawingFinished:
            return

        # self.setTransformOriginPoint(self.boundingRect().center())
        if delta > 0:
            # self.setRotation(self.rotation() + 5)
            self.scene().undoRedo.undoableRotate(self.scene().undoStack, "Rotate", self, self.rotation() + 5)
        else:
            # self.setRotation(self.rotation() - 5)
            self.scene().undoRedo.undoableRotate(self.scene().undoStack, "Rotate", self, self.rotation() - 5)
        # self.update_painter_text_rect()
        # self.update()

    # ---------------------------------------------------------------------
    def changeBackgroundColorAlpha(self, delta):

        if self.tempTextItem:
            return

        col = shared.calculate_alpha(delta, QColor(self.arkaPlanRengi))

        # self.setBrush(self.arkaPlanRengi)
        # self.update()
        # self.scene().undoableSetItemBackgroundColorAlpha("Change item's background alpha", self, col)

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' background alpha")
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "_change item's background alpha",
                                                                      self, col)
            for c in self.childItems():
                c.changeBackgroundColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "change item's background alpha", self,
                                                                      col)

    # ---------------------------------------------------------------------
    def changeLineColorAlpha(self, delta):

        if self.tempTextItem:
            return

        col = shared.calculate_alpha(delta, self._pen.color())

        # self.scene().undoableSetLineOrTextColorAlpha("Change item's line alpha", self, col)
        if self.childItems():
            self.scene().undoStack.beginMacro("change items' line alpha")
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack,
                                                            "_change item's line alpha", self, col)
            for c in self.childItems():
                c.changeLineColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack,
                                                            "change item's line alpha", self, col)

    # ---------------------------------------------------------------------
    def changeTextColorAlpha(self, delta):

        if self.tempTextItem:
            return

        col = shared.calculate_alpha(delta, self.yaziRengi)

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' text alpha")
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack,
                                                            "_change item's text alpha", self, col)
            for c in self.childItems():
                c.changeTextColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack,
                                                            "change item's text alpha", self, col)

    # ---------------------------------------------------------------------
    def changeImageItemTextBackgroundColorAlpha(self, delta):
        # grup nesnesinden cagriliyor alt+shift
        if self.childItems():
            startedMacro = False
            for c in self.childItems():
                if c.type() == shared.IMAGE_ITEM_TYPE:
                    if not startedMacro:
                        self.scene().undoStack.beginMacro("change image items' text background alpha")
                        startedMacro = True
                    c.changeImageItemTextBackgroundColorAlpha(delta)
            if startedMacro:
                self.scene().undoStack.endMacro()

    # # ---------------------------------------------------------------------
    # def mouseDoubleClickEvent(self, event):
    #     self.editMode = not self.editMode
    #     self.update()
    #     super(PathItem, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):

        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            self.editMode = not self.editMode
            self.update()
            # super(PathItem, self).mouseDoubleClickEvent(event)
            return

        # eger cizim yapiyorken , yani daha sahneye resmi olarak eklemedik
        # o zaman cizimibitirmeden cift tiklarsak cokgen nesnesinin yazi yazma kutucugu aciliyor
        # ama sonrasi problem, o yuzden asagidaki yazi yazma ozelligini eger parent yoksa demekki sahneye eklenmemis
        # diye dusunerek aktif etmiyoruz.
        # AYRICA SANHE DOUBLE CLICK inde, normal text nesnesi eklemse ile ilgili durum kontrol ediliyor.
        if self.isDrawingFinished:
            if not self.tempTextItem:
                self.tempTextItem = TempTextItem(self.sceneWidth() - self.textPadding,
                                                 self._font,
                                                 self.yaziRengi,
                                                 parent=self)
                self.tempTextItem.setParentItem(self)

            self.tempTextItem.setPlainText(self.text())
            self.tempEskiText = self.text()
            self.setText("")
            self.tempTextItem.setRotation(360 - self.rotation())
            # self.tempTextItem.rotateWithOffset(360 - self.rotation())
            self.tempTextItem.setCenter(self.boundingRect().center())
            self.tempTextItem.setFocus()

            # !! burda lambda kullanmazsak, self.is_temp_text_item_empty ,self daha olsumadigi icin,
            # selfi null olarak goruyor ve baglanti kuramiyor.
            self.tempTextItem.textItemFocusedOut.connect(lambda: self.is_temp_text_item_empty())

        super(PathItem, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    @Slot()
    def is_temp_text_item_empty(self):
        tmpText = self.tempTextItem.toPlainText()
        self.scene().undoRedo.undoableItemSetText(self.scene().undoStack, "change text", self,
                                                  self.tempEskiText, tmpText)
        # if not tmpText:
        self.scene().removeItem(self.tempTextItem)
        self.tempTextItem.deleteLater()
        self.tempTextItem = None

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        if self.editMode:
            liste = []
            for i in range(self._path.elementCount()):
                liste.append(
                    abs(self._path.elementAt(i).x - event.pos().x()) + abs(self._path.elementAt(i).y - event.pos().y()))
            en_yakin_nokta_degeri = min(liste)
            if en_yakin_nokta_degeri < 7:
                self.secilen_nokta_idx = liste.index(en_yakin_nokta_degeri)
                self.secilen_nokta = self._path.elementAt(self.secilen_nokta_idx)
                # print(self.secilen_nokta.x, self.secilen_nokta.y)
                self.movedPointsList.append((self.secilen_nokta_idx, self.secilen_nokta.x, self.secilen_nokta.y))
                self.update()

        else:
            self._resizing = False  # we could use "self._resizingFrom = 0" instead, but self._resizing is more explicit.
            self._fixedResizePoint = None
            self._resizingFrom = None
            self._eskiRectBeforeResize = None

            self._eskiPosBeforeResize = self.pos()
            self._ilk_tik_event_pos = event.pos()
            # self._eskiPosBeforeResize = self.scenePos()

            if self.topLeftHandle.contains(event.pos()):
                self._resizing = True
                self._fixedResizePoint = self._rect.bottomRight()
                self._resizingFrom = 1
            elif self.topRightHandle.contains(event.pos()):
                self._resizing = True
                self._fixedResizePoint = self._rect.bottomLeft()
                self._resizingFrom = 2
            elif self.bottomRightHandle.contains(event.pos()):
                self._resizing = True
                self._fixedResizePoint = self._rect.topLeft()
                self._resizingFrom = 3
            elif self.bottomLeftHandle.contains(event.pos()):
                self._resizing = True
                self._fixedResizePoint = self._rect.topRight()
                self._resizingFrom = 4
            self._eskiRectBeforeResize = self._rect

        super(PathItem, self).mousePressEvent(event)

        # TODO: bu kozmetik amacli, sadece release de olunca gecikmeli seciyormus hissi oluyor
        if self.isSelected() and self.childItems():
            self.scene().select_all_children_recursively(self, cosmeticSelect=True)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        if self.editMode:
            if self.secilen_nokta:
                path = QPainterPath(self._path)
                path.setElementPositionAt(self.secilen_nokta_idx, event.pos().x(), event.pos().y())
                self._path.swap(path)

                self.update()
                return

        elif self._resizing:

            px = self._fixedResizePoint.x()
            py = self._fixedResizePoint.y()
            # mx = event.scenePos().x()
            # my = event.scenePos().y()
            mx = event.pos().x()
            my = event.pos().y()
            topLeft = QPointF(min(px, mx), min(py, my))  # top-left corner (x,y)
            bottomRight = QPointF(max(px, mx), max(py, my))  # bottom-right corner (x,y)
            # size = QSizeF(fabs(mx - px), fabs(my - py))
            # rect = QRectF(topLeft, size)

            rect = QRectF(topLeft, bottomRight)

            c = self._rect.center()

            # Alt Key - to resize around center.
            # if event.modifiers() & Qt.AltModifier:
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                rect.moveCenter(c)

            # ---------------------------------------------------------------------
            #  Ctrl Key - to keep aspect ratio while resizing.
            if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
                tl = self._rect.topLeft()
                tr = self._rect.topRight()
                br = self._rect.bottomRight()
                bl = self._rect.bottomLeft()
                c = self._rect.center()

                yeniSize = rect.size()
                eskiSize = self._rect.size()
                # eskiSize.scale(yeniSize, Qt.KeepAspectRatio)
                eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.AspectRatioMode.KeepAspectRatioByExpanding)
                # eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.KeepAspectRatio)

                # if not eskiSize.isNull():
                if not eskiSize.isEmpty():
                    self.yedekSize = QSizeF(eskiSize)

                else:
                    eskiSize = QSizeF(self.yedekSize)

                rect.setSize(eskiSize)
                h = rect.height()
                w = rect.width()

                if self._resizingFrom == 1:
                    rect.moveTopLeft(QPointF(br.x() - w, br.y() - h))

                if self._resizingFrom == 2:
                    rect.moveTopRight(QPointF(bl.x() + w, bl.y() - h))

                if self._resizingFrom == 3:
                    rect.moveBottomRight(QPointF(tl.x() + w, tl.y() + h))

                elif self._resizingFrom == 4:
                    rect.moveBottomLeft(QPointF(tr.x() - w, tr.y() + h))

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            # ---------------------------------------------------------------------
            # Shift Key - to make square (equals width and height)
            if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
                height = my - py
                if self._resizingFrom == 1:
                    rect.setCoords(px + height, py + height, px, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 2:
                    rect.setCoords(px, py + height, px - height, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 3:
                    rect.setCoords(px, py, px + height, py + height)
                    rect = rect.normalized()

                elif self._resizingFrom == 4:
                    rect.setCoords(px - height, py, px, py + height)
                    rect = rect.normalized()

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            # fark = event.pos() - self._ilk_tik_event_pos

            # self.setRect(rect)
            # self.setPos(newPos)
            # TODO : alt calismiyor move to 0,0 dan dolayi
            self._resize(rect)  # mouse release eventten gonderiyoruz undoya
            # self.setRect(rect)  # mouse release eventten gonderiyoruz undoya
            self.update_resize_handles()
            self.scene().parent().change_transform_box_values(self)
            # self.setPos(x, y)
            # return QGraphicsItem.mouseMoveEvent(self, event)

            return

        super(PathItem, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):

        super(PathItem, self).mouseReleaseEvent(event)

        if self.secilen_nokta:
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                self.secilen_noktalari_sil()
            else:
                self.secilen_noktalari_tasi()

            self.movedPointsList = []
            # TODO: bu yukarda for dongusu ve macro yokmus gib su an tek nesne varmis gibi secilen nokta none diyor
            # bir cok nokta secmek ozelligi eklenirse bunu degistirmek lazim
            # self.secilen_nokta = None
            # self.update()

        elif self._resizing:

            devam, scaledPath, yeniPos = self._yeni_path_ve_pos_hesapla(yeniRect=self._rect)
            if devam:
                # self.setPos(yeniPos)
                # self.setPath(scaledPath)
                self.scene().undoRedo.undoableResizePathItem(self.scene().undoStack,
                                                             "resize",
                                                             self,
                                                             scaledPath,
                                                             yeniPos)
            self._resizing = False

            # super(PathItem, self).mouseReleaseEvent(event)

            # self.update_painter_text_rect()

        self.scene().unite_with_scene_rect(self.sceneBoundingRect())

        if self.childItems():
            if self.isSelected():
                self.scene().select_all_children_recursively(self, cosmeticSelect=False, topmostParent=True)
            else:
                self.scene().select_all_children_recursively(self, cosmeticSelect=False)

    # ---------------------------------------------------------------------
    def _yeni_path_ve_pos_hesapla(self, yeniRect, eskiRect=None,
                                  scaleFactorX=None,
                                  scaleFactorY=None,
                                  merkez=False):
        try:
            if not eskiRect:
                eskiRect = self._eskiRectBeforeResize

            if not scaleFactorX:
                scaleFactorX = yeniRect.width() / eskiRect.width()
            if not scaleFactorY:
                scaleFactorY = yeniRect.height() / eskiRect.height()

            yeniPos = self.mapToScene(yeniRect.center())
            if self.parentItem():
                yeniPos = self.mapToParent(yeniRect.center())

            scaleMatrix = QTransform()
            scaleMatrix.scale(scaleFactorX, scaleFactorY)
            scaledPath = self._path * scaleMatrix

            if merkez:
                fark = self.scenePos() - yeniPos
                scaledPath.translate(2 * fark)

            return True, scaledPath, yeniPos

        except ZeroDivisionError:
            # scaleFactorX = scaleFactorY = 1
            return False, None, None

    # ---------------------------------------------------------------------
    def _resize(self, yeniRect):
        self.setRect(yeniRect)
        # if self._rect == yeniRect:
        return

        # devam, scaledPath, yeniPos = self._yeni_path_ve_pos_hesapla(yeniRect=self._rect)
        # if devam:
        #     self.setPos(yeniPos)
        #     self.setPath(scaledPath)

    # ---------------------------------------------------------------------
    def secilen_noktalari_tasi(self):
        self.scene().undoStack.beginMacro("move {} point(s)".format(len(self.movedPointsList)))
        for secilen_nokta_idx, eskiX, eskiY in self.movedPointsList:
            self.scene().undoRedo.undoableMovePathPoint(undoStack=self.scene().undoStack,
                                                        description=self.scene().tr("Path point moved"),
                                                        item=self,
                                                        movedPointIndex=secilen_nokta_idx,
                                                        eskiPosTuple=(eskiX, eskiY),
                                                        yeniPosTuple=(self._path.elementAt(secilen_nokta_idx).x,
                                                                      self._path.elementAt(secilen_nokta_idx).y)
                                                        )
        self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def secilen_noktalari_sil(self):
        """Deneysel, ilk versiyon"""
        # TODO: bas ve son noktalar, ve de bunlara bagli degisebilecek isPathClosed durumu
        # print("noktalar", secilenNoktalar)

        # 2 for , slice ile, iflerden kurtulunulabilir
        secilenNoktalar = [idx[0] for idx in self.movedPointsList]
        yeniPath = QPainterPath()
        e0 = self._path.elementAt(0)
        yeniPath.moveTo(e0.x, e0.y)
        for i in range(self._path.elementCount()):
            if i not in secilenNoktalar:
                e = self._path.elementAt(i)
                yeniPath.lineTo(e.x, e.y)
        if self.isPathClosed:
            yeniPath.closeSubpath()
            self.setBrush(self.arkaPlanRengi)

        self.scene().undoRedo.undoableDeletePathPoint(undoStack=self.scene().undoStack,
                                                      description=self.scene().tr("Path point deleted"),
                                                      item=self,
                                                      eskiPath=self._path,
                                                      yeniPath=yeniPath)

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):
        painter.setPen(self._pen)
        painter.setBrush(self._brush)
        painter.drawPath(self._path)

        # if self.pixmap.isNull():
        #     painter.setPen(self._pen)
        #     painter.setBrush(self._brush)
        #     painter.drawPath(self._path)
        # else:
        #     painter.drawPixmap(self.boundingRect().toRect(), self.pixmap)

        if self._text:
            # painter.setWorldMatrixEnabled(False)
            painter.save()
            painter.setFont(self._font)
            # painter.setPen(self.textPen)
            painter.setPen(self.yaziRengi)
            painter.translate(self.boundingRect().center())
            painter.rotate(-self.rotation())
            painter.translate(-self.boundingRect().center())
            # metrics = painter.fontMetrics()
            # text = metrics.elidedText(self.text(), Qt.ElideRight, self.boundingRect().width())
            # TODO: self.boundingRect().width() calismadi, sceneWidth() de biraz maliyetli, optimize edilebilir.
            # txt = metrics.elidedText(self._text, Qt.ElideRight, self.sceneWidth() / scale)
            # painter.drawText(self.boundingRect(), Qt.AlignCenter | Qt.AlignVCenter, txt)
            painter.drawText(self.painterTextRect, self._text, self.painterTextOption)
            painter.restore()
            # painter.setWorldMatrixEnabled(True)

        if self.intersects:
            pen = QPen(self._pen)
            pen.setColor(self.arkaPlanRengi)
            painter.setPen(pen)
            painter.setBrush(self.arkaPlanRengi)
            # self.startPointRect.setSize(self.startPointRect.size() * self._pen.width())
            painter.drawEllipse(self.startPointRect)

        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:

            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
                if not self.isPinned:
                    painter.setPen(Qt.PenStyle.NoPen)
                    painter.setBrush(self.handleBrush)
                    painter.drawRect(self.topLeftHandle)
                    painter.drawRect(self.topRightHandle)
                    painter.drawRect(self.bottomRightHandle)
                    painter.drawRect(self.bottomLeftHandle)
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            # if self.pixmap.isNull():
            if self.editMode:
                painter.setPen(selectionPenBottom)
                painter.drawPath(self._path)

                # painter.setPen(self.selectionPenTop)
                # painter.drawPath(self._path)

                path = self._path
                # painter.setPen(QPen(self.cizgiRengi, 5))
                painter.setPen(QPen(Qt.GlobalColor.white, 5))
                for i in range(path.elementCount()):
                    painter.drawPoint(QPointF(path.elementAt(i).x, path.elementAt(i).y))
                if self.secilen_nokta:
                    painter.setPen(QPen(Qt.GlobalColor.red, 10))
                    painter.drawPoint(
                        QPointF(path.elementAt(self.secilen_nokta_idx).x, path.elementAt(self.secilen_nokta_idx).y))

            else:
                painter.setPen(selectionPenBottom)
                painter.drawRect(self._rect)

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     painter.drawPath(self._path)

        # painter.setPen(self.selectionPenTop)
        # painter.drawRect(self.boundingRect())

        # if self.editMode:
        #     # for a future release, draw points
        #     path = self._path
        #     painter.setPen(QPen(self.cizgiRengi, 10))
        #     for i in range(path.elementCount()):
        #         painter.drawPoint(QPointF(path.elementAt(i).x, path.elementAt(i).y))

        # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self.boundingRect(), "{},  {}\n{},  {}".format(p.x(), p.y(), s.x(), s.y()))
        # # painter.setPen(QPen(Qt.green, 12))
        # painter.setPen(QPen(Qt.red, 16))
        # painter.drawPoint(self.mapFromScene(p))
        # painter.setPen(QPen(Qt.blue, 12))
        # painter.drawPoint(self.mapFromScene(s))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # painter.drawRect(self.boundingRect())
        # painter.drawRect(p.x(),p.y(),100,100)
        # painter.drawRect(s.x(),s.y(),100,100)
        # painter.drawRect(0,0,15,15)
        # painter.drawRect(self._path.controlPointRect())
        # painter.drawRect(self.boundingRect())
        # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):

        rect = self.sceneBoundingRect()
        w = rect.width()
        h = rect.height()

        x = rect.left()
        y = rect.top()

        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = x - xs
        y = y - ys

        buffer = QBuffer()
        buffer.open(QIODevice.OpenModeFlag.WriteOnly)

        generator = QSvgGenerator()
        # generator.setFileName("dosya.svg")
        generator.setOutputDevice(buffer)
        # generator.setResolution(72)

        # generator.setSize(QSize(w, h))  # kaymalara sebep oluyor
        generator.setViewBox(QRectF(-w / 2, -h / 2, w, h))
        generator.setTitle(self._kim)
        generator.setDescription("")
        # painter = QPainter(generator)
        painter = QPainter()
        painter.begin(generator)
        painter.setPen(self._pen)
        painter.setBrush(self._brush)

        painter.rotate(self.rotation())
        painter.drawPath(self._path)
        painter.rotate(-self.rotation())

        if self._text:
            # painter.setWorldMatrixEnabled(False)
            painter.save()
            painter.setFont(self._font)
            # painter.setPen(self.textPen)
            painter.setPen(self.yaziRengi)
            painter.translate(self.boundingRect().center())
            painter.rotate(-self.rotation())
            painter.translate(-self.boundingRect().center())
            painter.drawText(self.painterTextRect, self._text, self.painterTextOption)
            painter.restore()
            # painter.setWorldMatrixEnabled(True)

        painter.end()

        svg_string = buffer.data().data().decode("utf-8")

        # background: rgba{self.arkaPlanRengi.toTuple()};\n
        div_str = f"""
                    <div style="
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     width:{w}px;
                     height:{h}px;
                     top:{y}px;
                     left:{x}px;" id="{self._kim}">{svg_string}</div>\n

            """

        # return svg_string
        return div_str



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '15/2/23'
__author__ = 'E. Y.'

from PySide6.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QPushButton, QApplication, \
    QLabel, QCheckBox
from PySide6.QtCore import Qt, Slot, QSettings, Signal
from . import shared
from .spinBoxlar import SpinBox


########################################################################
class PdfyiResmeCevirPenceresi(QDialog):
    # genislik, yukseklik, ilk, son, sayfalar arasi bosluk
    donusturTiklandi = Signal(int, int, int, int, int)

    # ---------------------------------------------------------------------
    def __init__(self, boyut_point, pdf_sayfa_sayisi, parent=None):
        super(PdfyiResmeCevirPenceresi, self).__init__(parent)

        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setWindowFlags(Qt.WindowType.WindowStaysOnTopHint)
        self.setWindowTitle(self.tr("Defter: PDF to Image Settings"))
        self.setModal(True)
        layout = QVBoxLayout(self)

        genislik = boyut_point.width() if boyut_point.width() else 1
        yukseklik = boyut_point.height() if boyut_point.height() else 1
        self.oran = genislik / yukseklik

        lblOraniKoru = QLabel(self.tr("Keep aspect ratio"), self)
        self.oraniKoruCBox = QCheckBox(self)
        self.oraniKoruCBox.setChecked(True)
        layOraniKoru = QHBoxLayout()
        layOraniKoru.addWidget(lblOraniKoru)
        layOraniKoru.addWidget(self.oraniKoruCBox)

        lblGenislik = QLabel(self.tr("Width"), self)
        self.genislikSBox = SpinBox(self)
        self.genislikSBox.setMinimum(1)
        self.genislikSBox.setMaximum(9999)
        self.genislikSBox.setValue(genislik)
        self.genislikSBox.valueChangedFromSpinBoxGuiNotBySetValue.connect(self.act_genislik_degisti)
        layGenislik = QHBoxLayout()
        layGenislik.addWidget(lblGenislik)
        layGenislik.addWidget(self.genislikSBox)

        lblYukseklik = QLabel(self.tr("Height"), self)
        self.yukseklikSBox = SpinBox(self)
        self.yukseklikSBox.setMinimum(1)
        self.yukseklikSBox.setMaximum(9999)
        self.yukseklikSBox.setValue(yukseklik)
        self.yukseklikSBox.valueChangedFromSpinBoxGuiNotBySetValue.connect(self.act_yukseklik_degisti)
        layYukseklik = QHBoxLayout()
        layYukseklik.addWidget(lblYukseklik)
        layYukseklik.addWidget(self.yukseklikSBox)

        lblIlkSayfa = QLabel(self.tr("From page"), self)
        self.ilkSayfaSBox = SpinBox(self)
        self.ilkSayfaSBox.setMinimum(1)
        self.ilkSayfaSBox.setMaximum(pdf_sayfa_sayisi)
        self.ilkSayfaSBox.valueChangedFromSpinBoxGuiNotBySetValue.connect(self.act_ilk_sayfa_degisti)
        layIlkSayfa = QHBoxLayout()
        layIlkSayfa.addWidget(lblIlkSayfa)
        layIlkSayfa.addWidget(self.ilkSayfaSBox)

        lblSonSayfa = QLabel(self.tr("To page"), self)
        self.sonSayfaSBox = SpinBox(self)
        self.sonSayfaSBox.setMinimum(1)
        self.sonSayfaSBox.setMaximum(pdf_sayfa_sayisi)
        self.sonSayfaSBox.setValue(pdf_sayfa_sayisi)
        self.sonSayfaSBox.valueChangedFromSpinBoxGuiNotBySetValue.connect(self.act_son_sayfa_degisti)
        laySonSayfa = QHBoxLayout()
        laySonSayfa.addWidget(lblSonSayfa)
        laySonSayfa.addWidget(self.sonSayfaSBox)

        lblSayfaArasiBosluk = QLabel(self.tr("Gap bw. pages"), self)
        self.sayfaArasiBoslukSBox = SpinBox(self)
        self.sayfaArasiBoslukSBox.setMinimum(0)
        self.sayfaArasiBoslukSBox.setMaximum(1000)
        self.sayfaArasiBoslukSBox.setValue(10)
        laysayfaArasiBosluk = QHBoxLayout()
        laysayfaArasiBosluk.addWidget(lblSayfaArasiBosluk)
        laysayfaArasiBosluk.addWidget(self.sayfaArasiBoslukSBox)

        donusturBtn = QPushButton(self.tr("C&onvert"), self)
        donusturBtn.clicked.connect(self.act_donustur)
        kapatBtn = QPushButton(self.tr("&Close"), self)
        kapatBtn.setDefault(True)
        kapatBtn.clicked.connect(self.reject)
        layBtn = QHBoxLayout()
        layBtn.addWidget(donusturBtn)
        layBtn.addWidget(kapatBtn)

        layout.addLayout(layOraniKoru)
        layout.addLayout(layGenislik)
        layout.addLayout(layYukseklik)
        layout.addLayout(layIlkSayfa)
        layout.addLayout(laySonSayfa)
        layout.addLayout(laysayfaArasiBosluk)
        layout.addLayout(layBtn)

        self.olustur_ayarlar()
        self.oku_ayarlar()
        # commandDialog.accepted.connect()

    # ---------------------------------------------------------------------
    @Slot(int)
    def act_ilk_sayfa_degisti(self, deger):
        self.sonSayfaSBox.setMinimum(deger)

    # ---------------------------------------------------------------------
    @Slot(int)
    def act_son_sayfa_degisti(self, deger):
        self.ilkSayfaSBox.setMaximum(deger)

    # ---------------------------------------------------------------------
    @Slot(int)
    def act_genislik_degisti(self, deger):
        if self.oraniKoruCBox.isChecked():
            self.yukseklikSBox.setValue(self.genislikSBox.value() / self.oran)

    # ---------------------------------------------------------------------
    @Slot(int)
    def act_yukseklik_degisti(self, deger):
        if self.oraniKoruCBox.isChecked():
            self.genislikSBox.setValue(self.yukseklikSBox.value() * self.oran)

    # ---------------------------------------------------------------------
    @Slot()
    def act_donustur(self):
        self.donusturTiklandi.emit(self.genislikSBox.value(),
                                   self.yukseklikSBox.value(),
                                   self.ilkSayfaSBox.value()-1,
                                   self.sonSayfaSBox.value()-1,
                                   self.sayfaArasiBoslukSBox.value()
                                   )
        self.accept()

    # ---------------------------------------------------------------------
    def closeEvent(self, event):
        self.yaz_ayarlar()
        event.accept()

    # ---------------------------------------------------------------------
    def olustur_ayarlar(self):
        QApplication.setOrganizationName(shared.DEFTER_ORG_NAME)
        QApplication.setOrganizationDomain(shared.DEFTER_ORG_DOMAIN)
        QApplication.setApplicationName(shared.DEFTER_APP_NAME)
        self.settings = QSettings(shared.DEFTER_AYARLAR_DOSYA_ADRES, QSettings.Format.IniFormat)
        # self.settings.clear()

    # ---------------------------------------------------------------------
    def oku_ayarlar(self):
        self.settings.beginGroup("PdfyiResmeCevirPenceresi")
        if self.settings.contains("CWinGeometry"):
            self.restoreGeometry(self.settings.value("CWinGeometry"))
        self.settings.endGroup()

    # ---------------------------------------------------------------------
    def yaz_ayarlar(self):
        self.settings.beginGroup("PdfyiResmeCevirPenceresi")
        self.settings.setValue("CWinGeometry", self.saveGeometry())
        self.settings.endGroup()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '10/8/23'
__author__ = 'E. Y.'

import sys

from PySide6.QtCore import QCoreApplication, QSize
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QPushButton, QStylePainter, QStyle, QStyleOptionButton, QFrame, QHBoxLayout, QApplication, QSizePolicy


#######################################################################
class DikeyDugme(QPushButton):
    def __init__(self, text, parent, yon="asagi"):
        super(DikeyDugme, self).__init__(text, parent)
        self.yon = yon
        # if self.yon == "sag":
        #     self.dondurme_acisi = 270
        #     self.kayma = (-1 * self.height(), 0)
        # elif self.yon == "sol":
        #     self.dondurme_acisi = 90
        #     self.kayma = (0, -1 * self.width())

        self.ufakBoyut = QSize(16, 75)
        self.normalBoyut = QSize(16, 100)
        # self.setFixedSize(self.normalBoyut)
        self.icerikW = None
        # self.show()

        # self.btnYuzdur = QPushButton("~", self)
        # self.btnYuzdur.setFixedSize(20, 20)
        # self.btnYuzdur.setFlat(True)
        # self.btnYuzdur.setAutoFillBackground(True)

        self.setFlat(True)
        # self.setAutoFillBackground(True)
        self.setContentsMargins(0, 0, 0, 0)
        # self.setAutoFillBackground(True)

        # self.renk_degistir(acik_mi=True)

        self.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        font = self.font()
        font.setPointSizeF(7)
        self.setFont(font)

        # self.setCheckable(True)
        # self.setChecked(False)

    # ---------------------------------------------------------------------
    def renk_degistir(self, acik_mi=True):

        if acik_mi:
            stil = """
            QPushButton{
                color:#fff;
                background-color: #99aabb;
                background-color: #aabcee;
                border: 0;
                padding: 3px;
                border-radius: 3px;
            }
            QPushButton:hover{
                background-color: #a4b6c8;
            }

            QPushButton:checked{
                background-color: #ced5e1;

            }
            QPushButton:checked:hover{
                background-color: #ced5e1;
            }

            """
        else:
            stil = """
            QPushButton{
                color:#5b5b5b;
                background-color: #bcbcbc;
                background-color: #aaa;
                border: 0;
                padding: 3px;
                border-radius: 3px;
            }
            QPushButton:hover{
                background-color: #ababab;
            }

            """
            # QPushButton:pressed{
            # background-color: #afc3d6;
            # }

        self.setStyleSheet(stil)

    # ---------------------------------------------------------------------
    def renk_degistir_belki(self, acik_mi=True):

        if acik_mi:
            self.renkYazi = QColor(135, 135, 135)
            self.renkArkaplan = QColor(250, 250, 250)
        else:
            self.renkYazi = QColor(165, 165, 165)
            self.renkArkaplan = QColor(220, 220, 220)
            # self.renkArkaplan2 = QColor(138, 165, 195)

        self.renkYazi2 = QColor(125, 125, 125)
        self.renkArkaplan2 = QColor(158, 185, 215)
        self.renkArkaplan2 = QColor(255, 255, 255)

        p = self.palette()
        p.setColor(self.foregroundRole(), self.renkYazi)
        p.setColor(self.backgroundRole(), self.renkArkaplan)
        self.setPalette(p)
        # p.setColor(self.btnYuzdur.foregroundRole(), self.renkYazi2)
        # p.setColor(self.btnYuzdur.backgroundRole(), self.renkArkaplan2)
        # self.btnYuzdur.setPalette(p)

    # ---------------------------------------------------------------------
    def renk_degistir_ilk(self, acik_mi=True):

        if acik_mi:
            self.renkYazi = QColor(255, 255, 255)
            self.renkArkaplan = QColor(153, 170, 187)
        else:
            self.renkYazi = QColor(190, 190, 190)
            self.renkArkaplan = QColor(133, 150, 167)
            # self.renkArkaplan2 = QColor(138, 165, 195)

        self.renkYazi2 = QColor(255, 255, 255)
        self.renkArkaplan2 = QColor(158, 185, 215)

        p = self.palette()
        p.setColor(self.foregroundRole(), self.renkYazi)
        p.setColor(self.backgroundRole(), self.renkArkaplan)
        self.setPalette(p)
        # p.setColor(self.btnYuzdur.foregroundRole(), self.renkYazi2)
        # p.setColor(self.btnYuzdur.backgroundRole(), self.renkArkaplan2)
        # self.btnYuzdur.setPalette(p)

    # ---------------------------------------------------------------------
    def paintEvent(self, event):
        painter = QStylePainter(self)
        if self.yon == "yukari":
            painter.rotate(270)
            painter.translate(-1 * self.height(), 0)
        if self.yon == "asagi":
            painter.rotate(90)
            painter.translate(0, -1 * self.width())
        painter.drawControl(QStyle.ControlElement.CE_PushButton, self.getSyleOptions())

    # ---------------------------------------------------------------------
    def minimumSizeHint(self):
        size = super(DikeyDugme, self).minimumSizeHint()
        size.transpose()
        return size

    # ---------------------------------------------------------------------
    def sizeHint(self):
        size = super(DikeyDugme, self).sizeHint()
        size.transpose()
        return size

    # ---------------------------------------------------------------------
    def getSyleOptions(self):
        options = QStyleOptionButton()
        options.initFrom(self)
        size = options.rect.size()
        size.transpose()
        options.rect.setSize(size)
        options.features = QStyleOptionButton.ButtonFeature.None_
        if self.isFlat():
            options.features |= QStyleOptionButton.Flat
        if self.menu():
            options.features |= QStyleOptionButton.HasMenu
        if self.autoDefault() or self.isDefault():
            options.features |= QStyleOptionButton.AutoDefaultButton
        if self.isDefault():
            options.features |= QStyleOptionButton.DefaultButton
        if self.isDown() or (self.menu() and self.menu().isVisible()):
            options.state |= QStyle.StateFlag.State_Sunken
        if self.isChecked():
            options.state |= QStyle.StateFlag.State_On
        if not self.isFlat() and not self.isDown():
            options.state |= QStyle.StateFlag.State_Raised

        options.text = self.text()
        options.icon = self.icon()
        options.iconSize = self.iconSize()
        return options

    # # ---------------------------------------------------------------------
    # def enterEvent(self, event):
    #     super(DikeyDugme, self).enterEvent(event)
    #     self.setMinimumWidth(20)
    #
    # # ---------------------------------------------------------------------
    # def leaveEvent(self, event):
    #     super(DikeyDugme, self).leaveEvent(event)
    #     self.setMinimumWidth(5)

    # # ---------------------------------------------------------------------
    # def mousePressEvent(self, event):
    #     super(DikeyDugme, self).mousePressEvent(event)
    #
    #     self._mousePressPosG = None
    #     self._mouseMovePosG = None
    #     if event.button() == Qt.MouseButton.LeftButton:
    #         self._mousePressPosG = event.globalPosition().toPoint()
    #         self._mouseMovePosG = self._mousePressPosG

    def mouseMoveEvent(self, event):
        super(DikeyDugme, self).mouseMoveEvent(event)

        # if event.buttons() == Qt.MouseButton.LeftButton:
        #     # adjust offset from clicked point to origin of widget
        #     eskiPos = self.pos()
        #     eskiPosMappedToGlobal = self.mapToGlobal(eskiPos)
        #     eskiPosGlobal = event.globalPosition().toPoint()
        #     fark = eskiPosGlobal - self._mouseMovePosG
        #     yeniPos = self.mapFromGlobal(eskiPosMappedToGlobal + fark)
        #     self.move(eskiPos.x(), yeniPos.y())
        #
        #     self._mouseMovePosG = eskiPosGlobal

        # if event.buttons() == Qt.MouseButton.RightButton:

        # if event.buttons() == Qt.MouseButton.LeftButton:
        #     drag = QDrag(self)
        #     mime = QMimeData()
        #     drag.setMimeData(mime)
        #
        #     pixmap = QPixmap(self.size())
        #     self.render(pixmap)
        #     drag.setPixmap(pixmap)
        #
        #     drag.exec(Qt.DropAction.MoveAction)


#######################################################################
class Main(QFrame):
    def __init__(self):
        QFrame.__init__(self)

        self.application = QCoreApplication.instance()
        self.layout = QHBoxLayout()
        self.button = DikeyDugme("Merhaba", self, yon="asagi")
        self.button2 = DikeyDugme("Merhaba", self, yon="yukari")
        self.layout.addWidget(self.button)
        self.layout.addWidget(self.button2)
        self.setLayout(self.layout)


# ---------------------------------------------------------------------
if __name__ == '__main__':
    application = QApplication(sys.argv)
    application.main = Main()
    application.main.show()
    sys.exit(application.exec())



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '15/2/23'
__author__ = 'E. Y.'

from PySide6.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout, QProgressBar, QLabel


#######################################################################
class LutfenBekleyin(QWidget):

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(LutfenBekleyin, self).__init__(parent)

        lay = QHBoxLayout()
        lay.setContentsMargins(0, 0, 0, 0)
        layCubuk = QVBoxLayout()
        layCubuk.setContentsMargins(0, 4, 0, 0)
        self.setLayout(lay)
        self.yuzdeCubuk = QProgressBar(self)
        self.yuzdeCubuk.setMaximumWidth(200)
        self.yuzdeCubuk.setMaximumHeight(15)
        self.yuzdeCubuk.setMinimum(0)
        self.yuzdeCubuk.setMaximum(0)
        # font = self.yuzdeCubuk.font()
        # font.setPointSizeF(8)
        # self.yuzdeCubuk.setFont(font)

        etiket = QLabel(self.tr("Please wait"), self)
        # etiket.move(30, 0)
        lay.addStretch()
        lay.addWidget(etiket)
        layCubuk.addWidget(self.yuzdeCubuk)
        lay.addLayout(layCubuk)

    # ---------------------------------------------------------------------
    def yuzdeCubukEnCokDeger(self):
        return self.yuzdeCubuk.maximum()

    # ---------------------------------------------------------------------
    def belirsiz_yuzde_moduna_al(self):
        self.yuzdeCubuk.setMaximum(0)

    # ---------------------------------------------------------------------
    def belirli_yuzde_moduna_al(self, encok_deger: int):
        self.yuzdeCubuk.setValue(0)
        self.yuzdeCubuk.setMaximum(encok_deger)

    # ---------------------------------------------------------------------
    def simdiki_deger_gir(self, deger: int):
        self.yuzdeCubuk.setValue(deger)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '05/07/22'
__author__ = 'E. Y.'

from PySide6.QtGui import QPen, QFont, QColor, Qt, QPixmap, QCursor


# ---------------------------------------------------------------------
def ver_arac_ikon_tipi(tip):
    tipIkonAdresSozluk = {'secimAraci': Qt.CursorShape.ArrowCursor,
                          'okAraci': ':icons/cursor-line.png',
                          'kutuAraci': ':icons/cursor-rectangle.png',
                          'yuvarlakAraci': ':icons/cursor-circle.png',
                          'kalemAraci': ':icons/cursor-pen.png',
                          'yaziAraci': ':icons/cursor-text.png',
                          'resimAraci': ':icons/cursor-image.png',
                          'videoAraci': ":icons/cursor-pen.png",
                          'dosyaAraci': ":icons/cursor-pen.png",
                          'aynalaXAraci': Qt.CursorShape.ArrowCursor,
                          'aynalaYAraci': Qt.CursorShape.ArrowCursor,
                          'resimKirpAraci': Qt.CursorShape.CrossCursor,
                          }
    return tipIkonAdresSozluk[tip]


#######################################################################
class Arac(object):

    # ---------------------------------------------------------------------
    def __init__(self, tip):
        # self.tip = tip

        self.tip = tip

        self.ikonAdres = ver_arac_ikon_tipi(tip)
        self._imlec = None

        self.kalem = QPen(QColor(0, 0, 0),
                          0,
                          Qt.PenStyle.SolidLine,
                          Qt.PenCapStyle.FlatCap,
                          Qt.PenJoinStyle.RoundJoin)

        self.arkaPlanRengi = QColor(71, 177, 213)
        self.yaziRengi = QColor(0, 0, 0)
        # asagida property olarak var
        # self.cizgiRengi = kalem.color()

        self.yaziTipi = QFont()
        self.yaziHiza = Qt.AlignmentFlag.AlignLeft
        self.karakter_bicimi_sozluk = {"b": False,
                                       "i": False,
                                       "u": False,
                                       "s": False,
                                       "o": False,
                                       }

    # ---------------------------------------------------------------------
    def oku_ozellikler(self):
        return {
            "kalem": self.kalem,
            "yaziTipi": self.yaziTipi,
            "arkaPlanRengi": self.arkaPlanRengi,
            "yaziRengi": self.yaziRengi,
            "cizgiRengi": self.cizgiRengi,
            "yaziHiza": int(self.yaziHiza),
            "karakter_bicimi_sozluk": self.karakter_bicimi_sozluk,
        }

    # ---------------------------------------------------------------------
    def yaz_ozellikler(self, aracOzellikleriSozluk):
        self.kalem = aracOzellikleriSozluk["kalem"]
        self.yaziTipi = aracOzellikleriSozluk["yaziTipi"]
        self.arkaPlanRengi = aracOzellikleriSozluk["arkaPlanRengi"]
        self.yaziRengi = aracOzellikleriSozluk["yaziRengi"]
        self.cizgiRengi = aracOzellikleriSozluk["cizgiRengi"]
        self.yaziHiza = Qt.AlignmentFlag(int(aracOzellikleriSozluk["yaziHiza"]))
        self.karakter_bicimi_sozluk = aracOzellikleriSozluk["karakter_bicimi_sozluk"]

    # ---------------------------------------------------------------------
    @property
    def imlec(self):
        if not self._imlec:
            if type(self.ikonAdres) is str:
                self._imlec = QCursor(QPixmap(self.ikonAdres))
            else:  # Qt.CursorShape
                self._imlec = self.ikonAdres  # Qt.ArrowCursor gibi

        return self._imlec

    # ---------------------------------------------------------------------
    @property
    def arkaPlanSeffaflik(self):
        return self.arkaPlanRengi.alpha()

    # ---------------------------------------------------------------------
    @arkaPlanSeffaflik.setter
    def arkaPlanSeffaflik(self, alpha):
        self.arkaPlanRengi.setAlpha(alpha)

    # ---------------------------------------------------------------------
    @property
    def yaziSeffaflik(self):
        return self.yaziRengi.alpha()

    # ---------------------------------------------------------------------
    @yaziSeffaflik.setter
    def yaziSeffaflik(self, alpha):
        self.yaziRengi.setAlpha(alpha)

    # ---------------------------------------------------------------------
    @property
    def yaziBoyutu(self):
        return self.yaziTipi.pointSize()

    # ---------------------------------------------------------------------
    @yaziBoyutu.setter
    def yaziBoyutu(self, boyut):
        self.yaziTipi.setPointSize(boyut)

    # ---------------------------------------------------------------------
    @property
    def cizgiRengi(self):
        return self.kalem.color()

    # ---------------------------------------------------------------------
    @cizgiRengi.setter
    def cizgiRengi(self, renk):
        self.kalem.setColor(renk)

    # ---------------------------------------------------------------------
    @property
    def cizgiSeffaflik(self):
        return self.cizgiRengi.alpha()

    # ---------------------------------------------------------------------
    @cizgiSeffaflik.setter
    def cizgiSeffaflik(self, alpha):
        self.cizgiRengi.setAlpha(alpha)

    # ---------------------------------------------------------------------
    @property
    def cizgiKalinligi(self):
        return self.kalem.widthF()

    # ---------------------------------------------------------------------
    @cizgiKalinligi.setter
    def cizgiKalinligi(self, kalinlik):
        self.kalem.setWidthF(kalinlik)

    # ---------------------------------------------------------------------
    @property
    def cizgiTipi(self):
        return self.kalem.style()

    # ---------------------------------------------------------------------
    @cizgiTipi.setter
    def cizgiTipi(self, lineStyle):
        self.kalem.setStyle(lineStyle)

    # ---------------------------------------------------------------------
    @property
    def cizgiBirlesimTipi(self):
        return self.kalem.joinStyle()

    # ---------------------------------------------------------------------
    @cizgiBirlesimTipi.setter
    def cizgiBirlesimTipi(self, joinStyle):
        self.kalem.setJoinStyle(joinStyle)

    # ---------------------------------------------------------------------
    @property
    def cizgiUcuTipi(self):
        return self.kalem.capStyle()

    # ---------------------------------------------------------------------
    @cizgiUcuTipi.setter
    def cizgiUcuTipi(self, capStyle):
        self.kalem.setCapStyle(capStyle)
